/*
 * Core modules for the IBM OSS Distributed Lock Manager
 *
 * Copyright (C) 2001, International Business Machines
 *
 * Authors: Peter R. Badovinatz, Ian D. Romanick
 * Contacts: wombat@us.ibm.com, romanick@us.ibm.com
 * http://oss.software.ibm.com/developerworks/opensource/linux/projects/dlm
 *
 * This program is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY;  without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 * the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program;  if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
#ident "$Id: cccp_output.c,v 1.6 2001/03/27 20:51:31 idr Exp $"

/*
 *   COMPONENT_NAME: LIB
 *
 *   FUNCTIONS: cccp_send
 *		cccp_send_guts
 */

#if defined( __KERNEL__ )
# ifndef MODULE
#  define MODULE
# endif

# include <linux/kernel.h>
# define __NO_VERSION__
# include <linux/module.h>
# ifdef USING_FULL_VERSIONING
#  if CONFIG_MODVERSIONS==1
#   define MODVERSIONS
#   include <linux/modversions.h>
#  endif
# endif /* USING_FULL_VERSIONING */
#endif /* __KERNEL__ */

#include <cltrc.h>
#include <dlm_cluster.h>
#include <cccp.h>

#include "cccp_pkt.h"
#include "cccp_proto.h"
#include "cccp_private.h"


/* FUNCTION:	cccp_send*/
/**/
/* PURPOSE:	API function to send a message to another node/port, as*/
/*		defined in the spec.*/
/**/
/* INPUT:	msg	pointer to a clm_msg_t describing the message*/
/**/
/* OUTPUT:	CPERR_OK if packet sent, error code otherwise*/
/**/
/* NOTE:	Errors are reported by invoking the callback function*/
/*		specified in the clm_msg_t, if there is one.*/

static int
cccp_send_guts ( clm_msg_t *msg )
{
    if ( ! checkInit() ) 
    {
	return( CPERR_NOCCCP );
    }


    /* there is no node zero*/
    if ( (msg == NULL) || (msg->cm_node == 0) || (msg->cm_port == 0) ) {
	return( CPERR_ILLEGAL );
    }

    /* insanity check */
    if ( ! clm_node_reachable( msg->cm_node ) )
	return( CPERR_NODEDOWN );


    /* caller should probably init this, but we do it here */
    msg->cm_hops = 0;


    /* Deliver the message locally.  If localForward returns FALSE, then
     * the message still needs to be sent to a remote node.
     */

    msg->cm_packet = NULL;
    msg->cm_sender = thisNode();
    cccp_msg_deliver_local( msg, NULL,
		   (cccp_forward_callback_t *) cccp_msg_do_remote_send );

    return( CPERR_OK );
}

int
cccp_send ( clm_msg_t *msg )
{
    int status;

    bsdlog( LOG_DEBUG, "[%s:%s] enter, msg[%p] node[%d] port[%d]\n",
            PROCESS_NAME, __FUNCTION__,
            msg, msg->cm_node, msg->cm_port );

    TRCHKL3T(CPTRC_SEND_ENTRY,msg,msg->cm_node,msg->cm_port);
    
    status = cccp_send_guts(msg);

    bsdlog( LOG_DEBUG, "[%s:%s] exit, msg[%p] status[%d]\n",
            PROCESS_NAME, __FUNCTION__,
            msg, status );

    TRCHKL2T(CPTRC_SEND_EXIT,msg,status);
    return(status);
}
