/*
 * Core modules for the IBM OSS Distributed Lock Manager
 *
 * Copyright (C) 2001, International Business Machines
 *
 * Authors: Peter R. Badovinatz, Ian D. Romanick
 * Contacts: wombat@us.ibm.com, idr@us.ibm.com
 * http://oss.software.ibm.com/developerworks/opensource/linux/projects/dlm
 *
 * Copyright (C) 1991, 1992 Free Software Foundation, Inc.
 * This file is part of the GNU C Library.
 * 
 * The GNU C Library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * The GNU C Library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with the GNU C Library; see the file COPYING.LIB.  If
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave,
 * Cambridge, MA 02139, USA.
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
#ident "$Id: bsearch.c,v 1.3 2001/10/04 20:29:18 idr Exp $"

/* ============================================================ */
/*
 * bsearch adapted from GNU libc bsearch module.
 * bsearchp added for use in the DLM modules.
 */

/* ============================================================ */
#ifndef __KERNEL__
#define __KERNEL__
#endif
#ifndef MODULE
#define MODULE
#endif

#include <linux/kernel.h>
#define __NO_VERSION__                  /* Only dlm_kerndd.c sets version. */
#include <linux/module.h>
#include <linux/version.h>              /* Due to __NO_VERSION__ */
#ifdef USING_FULL_VERSIONING
#if CONFIG_MODVERSIONS==1
#define MODVERSIONS
#include <linux/modversions.h>
#endif
#endif /* USING_FULL_VERSIONING */
/* ============================================================ */

#include <linux/types.h>

#include "dlm.h"
#include "dlm_cluster.h"
#include "clmstructs.h"
#include "clm_data.h"		/* external data definitions */
#include "dlm_kernel.h"

#include "dlm_proto.h"  /* Should be LAST included file. */

/* ============================================================ */
/*
 * Perform a binary search for KEY in BASE which has
 * NMEMB elements of SIZE bytes each.  The comparisons
 * are done by (*COMPAR)().
 */

void *
bsearch(const void *key,
        const void *base,
        size_t nmemb,
        size_t size,
        int(*compar)(const void *, const void *))
{
  register size_t l, u, idx;
  register void *p;
  register int comparison;

  DBPRINTF((DBG_CLIENT,
            __FUNCTION__": key %x base %x num %d size %d\n",
            key, base, nmemb, size));

  l = 0;
  u = nmemb;
  while (l < u) {
    idx = (l + u) / 2;
    p = (void *) (((const char *) base) + (idx * size));
    comparison = (*compar)(key, p);
    if (comparison < 0)
      u = idx;
    else if (comparison > 0)
      l = idx + 1;
    else {
      DBPRINTF((DBG_CLIENT,
                __FUNCTION__": return %x\n",
                p));
      return (void *) p;
    }
  }

  DBPRINTF((DBG_CLIENT,
            __FUNCTION__": return NULL\n"));

  return NULL;
}

/*
 * Same as bsearch(), EXCEPT, if we do not find the entry then
 * we return a pointer to the entry just BEFORE the insertion
 * point (i.e., a new entry).  base[-1] returned if insertion
 * at beginning, NULL if entry is present.
 */
void *
bsearchp(const void *key,
         const void *base,
         size_t nmemb,
         size_t size,
         int(*compar)(const void *, const void *))
{
  register size_t l, u, idx;
  register void *p;
  register int comparison;

  DBPRINTF((DBG_CLIENT,
            __FUNCTION__": key %x base %x num %d size %d\n",
            key, base, nmemb, size));

  idx = 0;
  l = 0;
  u = nmemb;
  while (l < u) {
    idx = (l + u) / 2;
    p = (void *)(base + (idx * size));
    comparison = (*compar)(key, p);
    if (comparison < 0) {
      u = idx;
    }
    else if (comparison > 0) {
      idx++;
      l = idx;
    }
    else {
      DBPRINTF((DBG_CLIENT,
                __FUNCTION__": return NULL\n"));
      return NULL;
    }
  }

  p = (void *) (base + (idx * size) - size);
  DBPRINTF((DBG_CLIENT,
            __FUNCTION__": return %x\n",
            p));

  return (void *) p;
}
