/*
 * Core modules for the IBM OSS Distributed Lock Manager
 *
 * Copyright (C) 2001, International Business Machines
 *
 * Authors: Peter R. Badovinatz, Ian D. Romanick
 * Contacts: wombat@us.ibm.com, idr@us.ibm.com
 * http://oss.software.ibm.com/developerworks/opensource/linux/projects/dlm
 *
 * This program is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY;  without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 * the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program;  if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
#ident "$Id: clm_data.c,v 1.7 2001/09/20 23:01:48 idr Exp $"

/*
 *   COMPONENT_NAME: DLM
 *
 *   FUNCTIONS: none
 */

/*
 * BE SURE to add initialization code for anything in this file
 */

/* ============================================================ */
#ifndef __KERNEL__
#define __KERNEL__
#endif
#ifndef MODULE
#define MODULE
#endif

#include <linux/kernel.h>
#define __NO_VERSION__                  /* Only dlm_kerndd.c sets version. */
#include <linux/module.h>
#include <linux/version.h>              /* Due to __NO_VERSION__ */
#ifdef USING_FULL_VERSIONING
#if CONFIG_MODVERSIONS==1
#define MODVERSIONS
#include <linux/modversions.h>
#endif
#endif /* USING_FULL_VERSIONING */
/* ============================================================ */

#include <linux/types.h>

#include "dlm.h"
#include "ast.h"
#include "dlm_cluster.h"
#include "clmstructs.h"
#include "pti.h"
#include "dlm_kernel.h"

/* ============================================================ */

int dlm_debug;				/* enables debug stream if > 0 */
int local_debug;			/* pseudo host id */
int grace_period;

int kproc_pid ;				/* lock manager pid */
int uproc_pid ;				/* user mode proc who starts us */
int OurClusterID = -1;			/* cluster id */

/*
 * The following will be set based on command-line and/or
 * config file values, and will be sent into the kernel
 * module from the user-side daemon via a write.
 */
int res_seg_size;		/* size of resource segments */
int lock_seg_size;		/* lock table segment size */
int res_seg_count = 0;		/* number of res segs on primary */
int clm_reclocks_used ;         /* reclock structs currently in use */
int clm_reclocks_denied ;       /* # times lock denied because no more space */
int clm_reclocks_max ;          /* reclock structs allocated from lock_heap */
int clm_lock_warned ;
int clm_res_warned ;    /* # times warning sent for no more space for resources */
int clm_rescount ;      /* number of struct resource in use */
        /* sum of reclocks & resources must be < than this */
int clm_items_max = 0;
int kmem_cutoff = CLM_KMEM_LIMIT;


int maxresources = 0;		/* total resource table size */

#if defined(TIME_STATS)
struct lockstats global_stats;
#endif

int nclients = 0;		/* number of registered applications */

/*
 * Head and tail of the Deadlock Timeout Queue.  All reclocks that
 * are moved onto either the WAIT or CONVERT queue's are added to 
 * the end of this list.  All reclocks that are granted or canceled
 * are taken off this list.
 */

DECLARE_DLM_LIST( dl_timeout );

int nbatched = 0;     /* number of entries waiting to be xmitted */
int ninflight = 0;    /* number of entries xmitted but not returned */
int batchsize = 1;    /* send after this many pending */

clm_msg_t *bounce_list = NULL;

int clm_active = FALSE;	/* TRUE is lock manager is running */
