/*
 * Core modules for the IBM OSS Distributed Lock Manager
 *
 * Copyright (C) 2001, International Business Machines
 *
 * Authors: Peter R. Badovinatz, Ian D. Romanick
 * Contacts: wombat@us.ibm.com, idr@us.ibm.com
 * http://oss.software.ibm.com/developerworks/opensource/linux/projects/dlm
 *
 * This program is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY;  without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 * the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program;  if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
#ident "$Id: clm_locksw.c,v 1.3 2001/09/25 23:14:48 idr Exp $"

/*
 *   COMPONENT_NAME: DLM
 *
 *   FUNCTIONS: none
 */

/* ============================================================ */
#ifndef __KERNEL__
#define __KERNEL__
#endif
#ifndef MODULE
#define MODULE
#endif

#include <linux/kernel.h>
#define __NO_VERSION__                  /* Only dlm_kerndd.c sets version. */
#include <linux/module.h>
#include <linux/version.h>              /* Due to __NO_VERSION__ */
#ifdef USING_FULL_VERSIONING
#if CONFIG_MODVERSIONS==1
#define MODVERSIONS
#include <linux/modversions.h>
#endif
#endif /* USING_FULL_VERSIONING */
/* ============================================================ */

#include <linux/errno.h>
#include <linux/types.h>

#include "dlm.h"
#include "ast.h"
#include "dlm_cluster.h"
#include "clmstructs.h"
#include "pti.h"
#include "clm_data.h"
#include "dlm_kernel.h"
#include "clm_locksw.h"

#include "dlm_proto.h"  /* Should be LAST included file. */

/* ============================================================ */
/*
 * this table contains pointers to the routines that are
 * specific to each locking model. It is indexed by the 
 * clm_locktype field in the transaction structure
 */
const struct locksw lockswitch[] = {
  { 	/* UNIX */
	(enum clm_stats(*)(struct clm_msg*,int))master_lock_unix,
	begin_lock_unix,
    cont_lock_unix, 
	master_unlock_unix,
    begin_unlock_unix,
    cont_unlock_unix, 
	(void(*)(struct resource*,struct reclock*))wakeup_unix,
	purge_unix,
  },
  { 	/* VMS */
	(enum clm_stats(*)(struct clm_msg*,int))master_lock_clm,
    begin_lock_clm,
    cont_lock_clm, 
	master_unlock_clm,
    begin_unlock_clm,
    cont_unlock_clm, 
	wakeup_clm,
	purge_clm,
  }};

