#ifndef STATE_MACHINE_H
#define STATE_MACHINE_H

#include "hsm.h"
#include <clm_reconfig.h>

#ifdef IN_MACHINE
static hsm_msg_t const * clm_st_top( hsm_t * me, hsm_msg_t const * msg );

static hsm_msg_t const * clm_st_init( hsm_t * me, hsm_msg_t const * msg );
static hsm_msg_t const * clm_st_run( hsm_t * me, hsm_msg_t const * msg );
static hsm_msg_t const * clm_st_send_dir( hsm_t * me, hsm_msg_t const * msg );
static hsm_msg_t const * clm_st_drain( hsm_t * me, hsm_msg_t const * msg );
static hsm_msg_t const * clm_st_reconfig( hsm_t * me, hsm_msg_t const * msg );

static hsm_msg_t const * rc_init_wait( hsm_t * me, hsm_msg_t const * msg );
static hsm_msg_t const * rc_dir_init( hsm_t * me, hsm_msg_t const * msg );
static hsm_msg_t const * rc_barrier1( hsm_t * me, hsm_msg_t const * msg );
static hsm_msg_t const * rc_purge_wait( hsm_t * me, hsm_msg_t const * msg );
static hsm_msg_t const * rc_barrier2( hsm_t * me, hsm_msg_t const * msg );
static hsm_msg_t const * rc_rebuild_dir( hsm_t * me, hsm_msg_t const * msg );
static hsm_msg_t const * rc_barrier3( hsm_t * me, hsm_msg_t const * msg );
static hsm_msg_t const * rc_resource_query( hsm_t * me, hsm_msg_t const * msg );
static hsm_msg_t const * rc_barrier4( hsm_t * me, hsm_msg_t const * msg );
static hsm_msg_t const * rc_barrier5( hsm_t * me, hsm_msg_t const * msg );
static hsm_msg_t const * rc_barrier6( hsm_t * me, hsm_msg_t const * msg );
#endif



/* State name defines.  These defines link the entries in the state table
 * with the symbolic state names.
 */

#define CLM_ST_TOP (& dlm_recov_states[ 0 ])
#define CLM_ST_INIT (& dlm_recov_states[ 1 ])
#define CLM_ST_RUN (& dlm_recov_states[ 2 ])
#define CLM_ST_SEND_DIR (& dlm_recov_states[ 3 ])
#define CLM_ST_DRAIN (& dlm_recov_states[ 4 ])
#define CLM_ST_RECONFIG (& dlm_recov_states[ 5 ])
#define RC_INIT_WAIT (& dlm_recov_states[ 6 ])
#define RC_DIR_INIT (& dlm_recov_states[ 7 ])
#define RC_BARRIER1 (& dlm_recov_states[ 8 ])
#define RC_PURGE_WAIT (& dlm_recov_states[ 9 ])
#define RC_BARRIER2 (& dlm_recov_states[ 10 ])
#define RC_REBUILD_DIR (& dlm_recov_states[ 11 ])
#define RC_BARRIER3 (& dlm_recov_states[ 12 ])
#define RC_RESOURCE_QUERY (& dlm_recov_states[ 13 ])
#define RC_BARRIER4 (& dlm_recov_states[ 14 ])
#define RC_BARRIER5 (& dlm_recov_states[ 15 ])
#define RC_BARRIER6 (& dlm_recov_states[ 16 ])


#ifdef IN_MACHINE
const hsm_state_t dlm_recov_states[] =
{
    STATE_ASSIGN( NULL, clm_st_top, "CLM_ST_TOP" ),
    STATE_ASSIGN( CLM_ST_TOP, clm_st_init, "CLM_ST_INIT" ),
    STATE_ASSIGN( CLM_ST_TOP, clm_st_run, "CLM_ST_RUN" ),
    STATE_ASSIGN( CLM_ST_TOP, clm_st_send_dir, "CLM_ST_SEND_DIR" ),
    STATE_ASSIGN( CLM_ST_TOP, clm_st_drain, "CLM_ST_DRAIN" ),
    STATE_ASSIGN( CLM_ST_TOP, clm_st_reconfig, "CLM_ST_RECONFIG" ),
    STATE_ASSIGN( CLM_ST_INIT, rc_init_wait, "RC_INIT_WAIT" ),
    STATE_ASSIGN( CLM_ST_INIT, rc_dir_init, "RC_DIR_INIT" ),
    STATE_ASSIGN( CLM_ST_DRAIN, rc_barrier1, "RC_BARRIER1" ),
    STATE_ASSIGN( CLM_ST_DRAIN, rc_purge_wait, "RC_PURGE_WAIT" ),
    STATE_ASSIGN( CLM_ST_RECONFIG, rc_barrier2, "RC_BARRIER2" ),
    STATE_ASSIGN( CLM_ST_RECONFIG, rc_rebuild_dir, "RC_REBUILD_DIR" ),
    STATE_ASSIGN( CLM_ST_RECONFIG, rc_barrier3, "RC_BARRIER3" ),
    STATE_ASSIGN( CLM_ST_RECONFIG, rc_resource_query, "RC_RESOURCE_QUERY" ),
    STATE_ASSIGN( CLM_ST_RECONFIG, rc_barrier4, "RC_BARRIER4" ),
    STATE_ASSIGN( CLM_ST_RECONFIG, rc_barrier5, "RC_BARRIER5" ),
    STATE_ASSIGN( CLM_ST_RECONFIG, rc_barrier6, "RC_BARRIER6" ),
};
#else
extern const hsm_state_t dlm_recov_states[];
extern hsm_t * const dlm_recov_state_machine;
extern bool_t dlm_recov_in_reconfig;
extern bool_t dlm_recov_can_transact;
#endif

struct rc_msg
{
    hsm_msg_t    base;
    int   node;
    void      * data;
};

typedef struct rc_msg rc_msg_t;

enum 
{
    CLM_MSG_NODE_INIT = 1,
	CLM_MSG_NODE_UP,
	CLM_MSG_NODE_DOWN,
	CLM_MSG_NODE_ADD,
	CLM_MSG_NODE_DELETE,
	CLM_MSG_NODE_PURGE,
	RC_MSG_SEND_MORE = 64,
	RC_MSG_GOT_BARRIER,
	RC_MSG_PURGE_CHECK,
	RC_MSG_DIR_RETRY,
	RC_MSG_DIR_NAK,
	RC_MSG_DIR_NOT_UP,
	RC_MSG_DIR_RESTORE,
	RC_MSG_DIR_END,
	RC_MSG_DIR_DATA,
	RC_MSG_DIR_REQUEST,
	RC_MSG_RES_REQ,
	RC_MSG_RES_REQ_1ST,
	RC_MSG_RES_DATA,
	RC_MSG_RES_RESTART
};

#endif /* STATE_MACHINE_H */
