/*
 * Core modules for the IBM OSS Distributed Lock Manager
 *
 * Copyright (C) 2001, International Business Machines
 *
 * Authors: Peter R. Badovinatz, Ian D. Romanick
 * Contacts: wombat@us.ibm.com, idr@us.ibm.com
 * http://oss.software.ibm.com/developerworks/opensource/linux/projects/dlm
 *
 * This program is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY;  without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 * the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program;  if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

#ident "$Id: dlm_workqueue.c,v 1.7 2001/10/16 19:00:11 idr Exp $"

/* ============================================================ */
#ifndef __KERNEL__
#define __KERNEL__
#endif
#ifndef MODULE
#define MODULE
#endif

#include <linux/kernel.h>
#define __NO_VERSION__                  /* Only dlm_kerndd.c sets version. */
#include <linux/module.h>
#include <linux/version.h>              /* Due to __NO_VERSION__ */
#ifdef USING_FULL_VERSIONING
#if CONFIG_MODVERSIONS==1
#define MODVERSIONS
#include <linux/modversions.h>
#endif
#endif /* USING_FULL_VERSIONING */
/* ============================================================ */

#include <asm/errno.h>

#include "dlm_kernel.h"
#include "clmstructs.h"

#include "dlm_workqueue.h"

extern void dlm_start_shutdown( void );

/****************************************************************************/
/* dlm_workqueue_put_work - Add a work unit to the work-queue
 * 
 */

int
dlm_workqueue_put_work(
		        dlm_workqueue_t * work,
		        dlm_workunit_t * data
		      )
{
    spin_lock( & work->mutex );
    dlm_list_add_tail( & work->work_queue, (dlm_listnode_t *) data );
    spin_unlock( & work->mutex );

    up( & work->sema );

    return( 0 );
}




/****************************************************************************/
/* dlm_workqueue_get_work - Get a work unit from the work-queue
 * 
 */

dlm_workunit_t *
dlm_workqueue_get_work(
		        /* Work-queue from which a work-unit is to be
			 * retrieved.
			 */

		        dlm_workqueue_t * work,


		        /* If set to TRUE, the function will not block if
			 * the work-queue is empty.
			 */

		        bool_t  non_blocking
		      )
{
    dlm_listnode_t * node;
    bool_t           get_work;
    int              ret;

    if ( ! non_blocking )
    {
	ret = down_interruptible( & work->sema );
	if (-EINTR == ret) {
	    dlm_start_shutdown();
	}
	get_work = TRUE;
    }
    else
    {
	get_work = ( down_trylock( & work->sema ) == 0 );
    }

    node = NULL;
    if ( get_work )
    {
	spin_lock( & work->mutex );
	node = dlm_list_rem_head( & work->work_queue );
	spin_unlock( & work->mutex );
    }


    return( (dlm_workunit_t *) node );
}




/****************************************************************************/
/* dlm_workqueue_dispose_work_unit - Destroy a work-unit
 * 
 */

int
dlm_workunit_dispose(
		      dlm_workunit_t * unit
		    )
{
    int   status = 0;
    
    
    if ( unit->free_data != NULL )
    {
	status = unit->free_data( unit->data );
    }
    
    
    if ( status == 0 )
    {
	CLM_FREE( unit );
    }

    return( 0 );
}
