/*
 * Core modules for the IBM OSS Distributed Lock Manager
 *
 * Copyright (C) 2001, International Business Machines
 *
 * Authors: Peter R. Badovinatz, Ian D. Romanick
 * Contacts: wombat@us.ibm.com, idr@us.ibm.com
 * http://oss.software.ibm.com/developerworks/opensource/linux/projects/dlm
 *
 * This program is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY;  without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 * the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program;  if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
#ident "$Id: ast.h,v 1.9 2001/11/06 13:32:46 bkon Exp $"

/*
 *   COMPONENT_NAME: DLM
 *
 *   FUNCTIONS: none
 */

#ifndef AST_H
#define AST_H

#include "dlm.h"
#include <dlm_list.h>
#include "ast_stub.h"

/*
 * error status codes
 */
#define ASTE_BADHANDLE	1	/* bad AST handle 		*/
#define ASTE_NOASTS	2	/* resource allocation failure 	*/

/*
 | this table exists in kernel memory, if the AST is called more than
 | once before being delivered, the old data will be overwritten.
 | A slot is marked busy by having a non zero pid field.
 */
struct astinfo {
    dlm_listnode_t   link;
    struct lockstatus ast_lksb;	/* intermediate storage for passed data */
    struct lockstatus *ast_dest;/* destination for lksb in user space 	*/
    int ast_mode;		/* mode arg used by blocking ASTs 	*/
    dlm_astfunc_t * ast_funcp;	/* ast callback function 		*/
    void *ast_arg;		/* callback arg 			*/
};

/*
 | an AST handle contains enough information to defer the registration
 | of the AST (the AST slot will be allocated at a later time).
 */
struct ast_handle {
    int asth_pid;		/* process id 			*/
    int asth_tid;		/* thread id 			*/
    struct lockstatus *asth_dest;/* lksb (see astinfo)		*/
    dlm_astfunc_t * asth_funcp;	/* ast callback function 		*/
    void *asth_arg;		/* callback arg 		*/
    struct astinfo *asth_aptr;	/* index into AST table		*/
    struct clm_client *asth_client;	/* index client			*/
};

typedef struct ast_handle ast_handle_t;

#ifdef __KERNEL__

extern void ast_init(void);
extern void ast_cleanup( void );
extern int ASTcall_unlck( struct clm_client * cl, dlm_stats_t status,
    dlm_astfunc_t * ast_funcp, void *arg, void * extrap );
extern int ASTcall( struct ast_handle * hp, const struct lockstatus * lksbp,
    int mode );
extern int ASTpurge(int pid, int tid);
extern bool_t valid_handle(const struct ast_handle *hp);
extern int k_ASTget(struct poll_info *user_pi, int clpid);
extern void kernapi_k_ASTget();
extern void ASTfree( void * p );

#endif /* __KERNEL__ */

#endif /* AST_H */
