/*
 * Core modules for the IBM OSS Distributed Lock Manager
 *
 * Copyright (C) 2001, International Business Machines
 *
 * Authors: Peter R. Badovinatz, Ian D. Romanick
 * Contacts: wombat@us.ibm.com, idr@us.ibm.com
 * http://oss.software.ibm.com/developerworks/opensource/linux/projects/dlm
 *
 * This program is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY;  without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 * the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program;  if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
#ident "$Id: clm_data.h,v 1.6 2001/09/20 23:01:48 idr Exp $"

/*
 *   COMPONENT_NAME: CLLOCKD
 *
 *   FUNCTIONS: none
 */

#ifndef CLM_DATA_H
#define CLM_DATA_H

/* ============================================================ */
/*
 * Under AIX, this file was automatically generated from the
 * definitions in clm_data.c, using an ode tool called the
 * makeexterns script.  We don't have one of those, and I
 * cannot just copy it out of AIX, as it is not part of the
 * CLM code base, so I don't have permission.  In any case,
 * the definitions here MUST match the definitions in
 * clm_data.c, which only makes sense, but you need to manually
 * maintain the two, at least for now.
 */
/* ============================================================ */

#ifdef __KERNEL__
/*
 * BE SURE to add initialization code for anything in this file
 */

extern int dlm_debug;				/* enables debug stream if > 0 */
extern int local_debug;			/* pseudo host id */
extern int grace_period;

extern int kproc_pid ;				/* lock manager pid */
extern int uproc_pid ;				/* user mode proc who starts us */
extern int OurSiteID ;			/* lock manager site id */
extern int OurClusterID ;			/* cluster id */


/*
 * The following will be set based on command-line and/or
 * config file values, and will be sent into the kernel
 * module from the user-side daemon via a write.
 */
extern int res_seg_size;		/* size of resource segments */
extern int lock_seg_size;		/* lock table segment size */
extern int res_seg_count ;		/* number of res segs on primary */
extern int clm_reclocks_used ;         /* reclock structs currently in use */
extern int clm_reclocks_denied ;       /* # times lock denied because no more space */
extern int clm_reclocks_max ;          /* reclock structs allocated from lock_heap */
extern int clm_lock_warned ;
extern int clm_res_warned ;    /* # times warning sent for no more space for resources */
extern int clm_rescount ;      /* number of struct resource in use */
        /* sum of reclocks & resources must be < than this */
extern int clm_items_max ;
extern int kmem_cutoff ;

extern int maxresources ;		/* total resource table size */

#if defined(TIME_STATS)
extern struct lockstats global_stats;
#endif

extern int nclients ;		/* number of registered applications */

/* Deadlock Timeout Queue.  All reclocks that are moved onto either the
 * WAIT or CONVERT queue's are added to the end of this list.  All reclocks
 * that are granted or canceled are taken off this list.
 */

extern dlm_list_t dl_timeout;

extern int nbatched ;     /* number of entries waiting to be xmitted */
extern int ninflight ;    /* number of entries xmitted but not returned */
extern int batchsize ;    /* send after this many pending */

extern clm_msg_t *bounce_list ;

extern int clm_active ;	/* TRUE is lock manager is running */

#endif /*  __KERNEL__ */

#endif /* CLM_DATA_H */
