/*
 * Core modules for the IBM OSS Distributed Lock Manager
 *
 * Copyright (C) 2001, International Business Machines
 *
 * Authors: Peter R. Badovinatz, Ian D. Romanick
 * Contacts: wombat@us.ibm.com, idr@us.ibm.com
 * http://oss.software.ibm.com/developerworks/opensource/linux/projects/dlm
 *
 * This program is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY;  without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 * the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program;  if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
#ident "$Id: clm_migrate.h,v 1.2 2001/03/12 19:50:47 wombat Exp $"

/*
 *   COMPONENT_NAME: DLM
 *
 *   FUNCTIONS: none
 */

#ifndef CLM_MIGRATE_H
#define CLM_MIGRATE_H

#include "dlm_cluster.h"

/*
 | This header file contains data required by the resource migration
 | module.  Note that clmm is an acronym for Cluster Lock Manager Migration.
 */

/* Numeric Defines */
#define MAX_CELLS	(MAXNODES+1)	/* array size [cell 0 unused]	*/
#define MAX_LOCKS	15		/* number of reclocks in pkt	*/
#define MAX_STICK	100		/* maximum value for stickiness	*/
#define DEF_STICK	50		/* default value for stickiness	*/

/* Message Status Defines */
#define MIG_ACK		0x01		/* migration accepted		*/
#define MIG_NACK 	0x02		/* migration rejected		*/

/* Message Types Defines */
#define MIGA		0x01		/* migration acknowledgement	*/
#define MIGR		0x02		/* migration request		*/

/******* HACK !!!!!!!! *******/
#define MSYNC		0x99		/* SUBVERTED for in-band global */
					/* sync to old (4.1.1) lockmgrs */
					/* since it will not generate a */
					/* fatal error when received    */

/* Control Block Flag Defines */
#define MCB_EVNT	20		/* default event run size	*/
#define MCB_APSD	0.875		/* default decay rate		*/
#define MCB_DOWN	0x00		/* disable all migration	*/
#define MCB_SEND	0x01		/* outgoing migration enabled	*/
#define MCB_RCVE	0x02		/* incoming migration enabled	*/
#define MCB_MASK	(MCB_SEND|MCB_RCVE)

/*
 | global control block for migration.
 */
typedef struct clmm_cb {
    int mcb_state;			/* active, inactive, etc...	*/
    uint mcb_event_run;			/* number of events for recalc	*/
    uint64_t mcb_aps_decay;		/* decay rate for APS		*/
    struct resource *mcb_rcache;	/* cached resource pointer	*/
    int mcb_ncache;			/* cached node ID		*/
} clmm_cb_t;

/*
 | control ops.
 */
typedef enum clmm_ops {
    CLMM_SETGCB=1,			/* set CB from user to lockd	*/
    CLMM_GETGCB,			/* get CB from lockd to user	*/
    CLMM_UPDGCB,			/* partial set CB from lockd	*/
    CLMM_SETSTK,			/* set stickiness		*/
    CLMM_GETSTK,			/* get current stickiness	*/
    /* used for spec compatibility */
    SETS,				/* old set stickiness		*/
    GETS				/* old get current stickiness	*/
} clmm_ops_t;

/*
 | Resource counter block used to schedule migration.  This
 | structure should be contained within the resource struct
 | itself.
 */
typedef struct clmm_info {
    struct timeval mi_timestamp;	/* timestamp from last recalc	*/
    int mi_event_ttl;			/* running event total		*/
    u_long mi_stickiness;		/* stickiness for resource	*/
    int mi_counter[MAX_CELLS];		/* accesses per node in run	*/
    uint64_t mi_aps[MAX_CELLS];		/* normalized APS		*/
} clmm_info_t;

/*
 * version 4.1.1
 */
/* temp defines until structs are actually built */
#if 0
struct clmm_data_new	/* temp name, really 4.1.1 */
{
    int cmig_version;			/* protocol version     	*/
    int cmig_csize;			/* max number of nodes		*/
    int	cmig_flags;			/* state bits 			*/
    int cmig_q_num[RLQ_MAX] ;		/* length of grant/conv/wait qs	*/
    struct resource_public cmig_res ;	/* public contents of resource	*/
    struct resource *cmig_rptr;		/* cached pointer for response	*/

    /* This field should always be last! */
/* this is what it SHOULD say, but include nesting prevents this */
    struct reclock_public cmig_data[1] ;/* variadic array of reclocks	*/
} ;
#endif

/*
 | Version 3.1.1
 | clmm_data is a structure that holds all the data necessary
 | to migrate a resource's master from one node to another.
 */
/*
 * WARNING - examine compatibility code if the order or size changes !!!!
 * the array cmig_q_num is compatible with the 3 ints in older vers.
 */
typedef struct clmm_data {
    int cmig_version;			/* protocol version     	*/
    int cmig_csize;			/* max number of nodes		*/
    int	cmig_flags;			/* state bits 			*/
    int cmig_q_num[RLQ_MAX] ;		/* length of grant/conv/wait qs	*/
    char cmig_value[MAXLOCKVAL];	/* lock value block		*/
    clmm_info_t cmig_info;		/* migration stats		*/
    int cmig_cache;			/* bytes cmig_data to reclock	*/
    struct resource *cmig_rptr;		/* cached pointer for response	*/
    dlm_statistics_t cmig_stats;        /* resource statistics          */
    int cmig_lastnode;                  /* last node to make request    */

    /* This field should always be last! */
    void *cmig_data;			/* marker for varlen data	*/
} clmm_data_t;

/*
 | Version 3.1 for compatibility
 | clmm_data is a structure that holds all the data necessary
 | to migrate a resource's master from one node to another.
 */
typedef struct clmm_data31 {
    int cmig_version;			/* protocol version     	*/
    int cmig_csize;			/* max number of nodes		*/
    int	cmig_flags;			/* state bits 			*/
    int cmig_lockq_num;			/* length of grant queue 	*/
    int cmig_convq_num;			/* length of convert queue 	*/
    int cmig_waitq_num;			/* length of wait queue		*/
    char cmig_value[MAXLOCKVAL];	/* lock value block		*/
    clmm_info_t cmig_info;		/* migration stats		*/
    int cmig_cache;			/* bytes cmig_data to reclock	*/
    struct resource *cmig_rptr;		/* cached pointer for response	*/

    /* This field should always be last! */
    void *cmig_data;			/* marker for varlen data	*/
} clmm_data31_t;

/*
 | clmm_request_t is the structure passed to the migration
 | module from the user to set or retrieve the:
 | 1. control block
 | 2. stickiness
 */
typedef struct clmm_request {
    u_long stickiness;			/* stickiness value	*/
    int lockid;				/* lockid on resource	*/
} clmm_request_t;

/*
 | clmm_ack is the structure returned by the RMM peer on a
 | remote node in response to a migration request.
 */
typedef struct clmm_ack {
    int cma_version;			/* protocol version		*/
    int cma_csize;			/* maximum number of nodes	*/
    int cma_status;			/* response from peer		*/
    struct resource *cma_resptr;	/* pointer to the resource	*/
} clmm_ack_t;

#endif /* CLM_MIGRATE_H */
