/*
 * Core modules for the IBM OSS Distributed Lock Manager
 *
 * Copyright (C) 2001, International Business Machines
 *
 * Authors: Peter R. Badovinatz, Ian D. Romanick
 * Contacts: wombat@us.ibm.com, idr@us.ibm.com
 * http://oss.software.ibm.com/developerworks/opensource/linux/projects/dlm
 *
 * This program is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY;  without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 * the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program;  if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
#ident "$Id: clm_reconfig.h,v 1.4 2001/11/29 21:59:24 idr Exp $"

/*
 *   COMPONENT_NAME: DLM
 *
 *   FUNCTIONS: none
 */

#ifndef CLM_RECONFIG_H
#define CLM_RECONFIG_H

#include "dlm_cluster.h"

/* This header file contains data specific to the clm_reconfig module.  Note
 * that clmr is an acronym for Cluster Lock Manager Reconfiguration.
 */

/* Structures Used */

/* clmr_data is a structure that holds all the data necessary to migrate a
 * resource's and its locks from one node to another.
 */
typedef struct clmr_data {
    int creb_version;			/* protocol version     	*/
    int creb_csize;			/* max number of nodes		*/
    int	creb_flags;			/* state bits 			*/
    int creb_lockq_len[RLQ_MAX] ;	/* # items in grant/conv/wait qs */
    int creb_ref_num;			/* # of refs for UNIX locks	*/
    char creb_value[MAXLOCKVAL];	/* lock value block		*/
    char creb_name[MAXDLMNAMELEN];		/* resource name		*/
    int creb_namelen;			/* resource name length		*/
    dlm_restype_t creb_type;		/* resource type 		*/
    struct resource *creb_rptr;		/* cached pointer for response	*/

    /* This field should always be last! */
    void *creb_data;			/* marker for varlen data	*/
} clmr_data_t;

/* Resource merge acknowledgement */
typedef struct clr_res {
    struct resource *clr_resptr;		/* resource pointer	*/
    union dlm_rh clr_mst_rh;			/* master handle	*/
} clr_res_t;

/* Directory information summary */
typedef struct rl_summary {
    int rls_num;			/* Incoming directory entries	*/
    uint rls_event_run;			/* global migration params in	*/
    uint64_t rls_aps_decay;		/* clm_migrate.h		*/
} rl_summary_t;

#endif /* CLM_RECONFIG_H */

