/*
 * Core modules for the IBM OSS Distributed Lock Manager
 *
 * Copyright (C) 2001, International Business Machines
 *
 * Authors: Peter R. Badovinatz, Ian D. Romanick
 * Contacts: wombat@us.ibm.com, idr@us.ibm.com
 * http://oss.software.ibm.com/developerworks/opensource/linux/projects/dlm
 *
 * This program is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY;  without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 * the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program;  if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
#ident "$Id: clmstructs.h,v 1.20 2001/11/29 13:20:21 bkon Exp $"

/*
 *   COMPONENT_NAME: DLM
 *
 *   FUNCTIONS: DBPRINTF
 *		getstamp
 *		resptr
 *		same_proc
 *		timestamp
 */

#ifndef CLMSTRUCTS_H
#define CLMSTRUCTS_H

#ifndef __KERNEL__
#include <sys/types.h>
#include <sys/param.h>
#include <netinet/in.h>
#include <sys/time.h>
#include <unistd.h>
#else
#include <linux/types.h>
#include <linux/socket.h>
#include <linux/in.h>
#include <linux/limits.h>
#include <linux/time.h>
#include <linux/wait.h>
#endif /* __KERNEL__ */

#include "cccp.h"
#include "ast.h"

/* ============================================================ */

#define GRACE_SLEEP 3

/*
 * the following definitions are BEFORE the #include of clm_migrate.h
 * BECAUSE the migration structures and general structures cross-reference
 * in such a way that one set of definitions has to be pulled up.
 * (migrate structs hold queues of reclocks & resource structs hold
 *     migration statistics)
 */

/*
 * indices for the 3 queues of reclocks in the resource:
 *      grant (aka lock), convert, and wait
 * must start at 0 and can't be changed unless queue arch changes
 */
#define RLQ_GRANT 0
#define RLQ_CONV 1
#define RLQ_WAIT 2
#define RLQ_MAX 3

#include "clm_migrate.h"

#define MAXNAMLEN 	255
/*
 * This constant can't exceed 16 bits without changing the layout
 * of the lockstatus structure
 */
#define CLM_KMEM_LIMIT 	50		/* max percent of kmem allowed */
#define MAXRESSEGSIZE 	65535		/* max resource segment size */
#define	DEFRESSEGSIZE	8192		/* default resource segment size */
#define	MAXLOCKSEGSIZE	65535		/* max lock segment size */
#define	DEFLOCKSEGSIZE	16384		/* default lock segment size */
#define MAX_RES_SEGS 	256		/* max number of resource segments */
#define MAX_LOCK_SEGS 	256		/* max number of lock segments */
#define MAXCLMHANDLES	MAXRESSEGSIZE
#define DEFHASHSIZE	100000		/* default hash table size */
#define MAXHASHSIZE	2000000		/* maximum hash table size */
#define	TR_BUFF_LIMIT	16384		/* clm_trans_t allocation limit */

#define DEFDLMDUMPFILE "/tmp/haDLMdump" /* DLM default dump device */
/*
 * The device base name is used to determine the names of the:
 * - device entry (/dev/basename)
 * - proc entry (/proc/basename)
 */
#define DEFDLMCONFIGFILE "/etc/haDLM"   /* Default configuration file */
#define DEFDLMDEVICEBASE "haDLM"        /* DLM device name */
#define DEFDLMMAJORNUMBER 250           /* DLM major number */
#define DEFDLMADMINBASE "admin"         /* Admin device entry */
#define DEFDLMADMINMINOR 0              /* Admin device minor */
#define DEFDLMLOCKSBASE "locks"         /* Lock device entry */
#define DEFDLMLOCKSMINOR 1              /* Lock device minor */

#define DEFDLMCLUSTERMGR "rsct"         /* DLM Cluster Manager load module */

#define MAXLOCKS	65535
#define	MAXDOMAINS	2

#define DLM_PROG 	20000010	/* used by RPC and straight UDP */
#define DLM_NOPROG	0
#define DLM_VERS 	3
#define DLM_BATCHSIZE	32

/* locks of length zero get extended to "infinite length" */
#if !defined(MAXEND)
#define MAXEND		017777777777
#endif

#ifndef BOOL_T_DEFINED
#define BOOL_T_DEFINED
typedef int bool_t;
#endif /* BOOL_T_DEFINED */

/* debugging facilities */
#define DBG_BASIC	0x1	/* basic debugging level */
#define DBG_FUNC	0x2	/* function tracing */
#define DBG_CTI		0x4	/* CTI tracing */
#define DBG_PTI		0x8	/* PTI tracing */
#define DBG_DEADLOCK	0x10	/* deadlock tracing */
#define DBG_STATS	0x20	/* statistics tracing */
#define DBG_CLMLOCK	0x40	/* CLM lock specific tracing */
#define DBG_UNIXLOCK	0x80	/* UNIX lock specific tracing */
#define DBG_COMM	0x100	/* communication layer info */
#define DBG_RESOURCE	0x200	/* resource function tracing */
#define DBG_LOCK	0x400	/* common locking facilities */
#define DBG_FORCE	0x1000	/* output whenever debug is active */
#define DBG_BATCH	0x2000	/* PTI batch tracing */
#define DBG_AST		0x4000	/* AST tracing */
#define DBG_TIMEOUT	0x8000	/* lock timeout tracing */
#define DBG_DYNRES	0x10000	/* dynamic resource allocation tracing */
#define DBG_NETFAIL	0x20000	/* client process id code */
#define DBG_CONFIG	0x40000	/* message queue config code paths */
#define DBG_SCN		0x80000	/* SCN functions */
#define DBG_DIRECT	0x100000/* directory functions */
#define DBG_MIGR	0x200000/* dynamic remastering			*/
#define DBG_MIGR2	0x400000/* migration complete/resource location	*/
#define DBG_RECOV	0x800000/* node failure recovery		*/
#define DBG_TUNE	0x1000000/* migration tuning operation		*/
#define DBG_GROUP	0x2000000/* group tuning operation		*/
#define DBG_SYNC        0x4000000/* clmlock_sync call tracing */
#define DBG_LOCAL       0x8000000/* local lock tracing */
#define DBG_PURGE	0x10000000	/* client purge events */
#define DBG_KDBX	0x20000000	/* cause brkpoint() during startup */
#define DBG_FAILSYNC	0x40000000	/* clm_sync() paths */
#define DBG_CLIENT	0x80000000	/* client process id code */

/* dbprintf macro - all calls to dbprintf should have double ()'s */
#ifdef DBPRINT
#define DBPRINTF(arg) clm_dbprintf arg
#else
#define DBPRINTF(arg)
#endif /* DBPRINT */

#ifdef __KERNEL__
/* all kernel clients use same dummy pid, so different kernel threads can
   work as a group */

#define DLM_KERNEL_PID 0x7F7F7F7F
#define CH_NOREF	1	/* check_handles: do NOT check resource refcnt */
#endif

/* assert macro for the lock manager */
#if defined(DEBUG)
#define CLM_ASSERT(cond,text)	\
	do { if ( ! (cond) ) abort((text)); } while( 0 )

#else
#define CLM_ASSERT(cond,text)
#endif /* defined(DEBUG) */

#ifdef __KERNEL__
void abort( const char * );
#endif

/*
 * Operations that can be performed on groups.
 */
typedef enum {
	DLM_GRP_CREATE,
	DLM_GRP_ATTACH,
	DLM_GRP_DETACH,
	DLM_GRP_MAXOP
} dlm_grp_ops;

/*
 * these are the requests that can be sent to the PTI and CTI
 */
typedef enum dlm_ops {
	DLM_REGISTER=1,
	DLM_UNREGISTER,
	DLM_LOCK,
	DLM_UNLOCK,
	DLM_CONVERT,		/* convert a lock to a new mode */
	DLM_APPREG,		/* application registering with lockd */
	DLM_CONFIGCOMPLETE,	/* a new primary has been elected */
	DLM_PURGELOCKS,
	DLM_LOCK_DUMP,		/* send a lock to a new primary */
	DLM_RESEND,		/* sent to primary after network failure */
	DLM_RESOURCE_DUMP,	/* send a resource to a new primary */
	DLM_DUMPCOMPLETE,	/* all locks have been sent to the primary */
	DLM_STALL,		/* initiate a lock stall */
	DLM_UNSTALL,		/* terminate a lock stall */
	DLM_STATS,              /* get resource statistics */
	DLM_SCN_OP,		/* scn operation */
	DLM_RMIGR_OP,		/* change resource migration parameters */
	DLM_GMIGR_OP		/* change global migration parameters */
} dlm_ops_t;

/* these make XID refs easier */
#define	xid_c	xid_u.xid_char
#define	xid_i	xid_u.xid_int

/*
 * The following structure is used to represent a registered resource
 * All lock related queues hang off the resource
 */

struct resource {
	int	refcount;		/* reference count 		*/
	union dlm_rh rh;		/* resource handle 		*/
	union dlm_rh	mst_rh;		/* master resource handle       */
	dlm_restype_t type;		/* resource type 		*/
	char	*name;			/* dynamic resource name 	*/
	short 	namelen;		/* length of resource name 	*/
	short	master;			/* site id of master node       */
	dlm_list_t   lockqs[ RLQ_MAX ];	/* grant, convert, wait lists */
	char 	value[MAXLOCKVAL];	/* lock value block 		*/
	int 	rsrc_flags;		/* state bits 			*/
	clmm_info_t clminfo;		/* migration data 		*/
	int     lastnode;               /* nodeid of last request       */
	dlm_statistics_t stats;         /* resource statistics          */
	union {
		struct resource	*free_next;	/* next resource in freelist */
		struct resref *refs;		/* resource reference list */
	} reuse;
	void	*migr_ack;		/* pointer to migration ack message */
};

#define free_next	reuse.free_next
#define refs		reuse.refs
typedef int (resource_func_t )( struct resource *, void * );

/*
 * flags for resource flags field 
 */
#define RSRC_VALID	0x01	/* value block invalid	        */
#define RSRC_FRZEN	0x02	/* resource in-flight	        */
#define RSRC_LOCAL	0x04	/* resource create w/ LKM_LOCAL */
#define RSRC_STUCK	0x08	/* migration ACK not yet complete */
#define RSRC_UPDATING	0x10	/* migration dirupdt not yet complete */
#define RSRC_ACTIVE	0x20	/* set when in use, clear when free */
#define RSRC_NEW_MASTER	0x40	/* new master node assigned */

__inline__ static struct resource *
resptr( union dlm_rh hndl )
{
    struct resource * r;
    extern struct resource * resource_get( unsigned, unsigned );


    r = resource_get( hndl.handle.seg, hndl.handle.cookie );
    if ( (r != NULL) && ((r->rsrc_flags & RSRC_ACTIVE) == 0) )
    {
	r = NULL;
    }
    
    return( r );
}


/*
 * resource registration request structure
 * one of these gets passed to alocate_resource() and contains all the
 * information needed to create a new resource
 */
struct clm_regreq {
	char   		*name;
	short		namelen;
	short		master;
	dlm_restype_t   type;
	int		pid;
	int		site;
	int		count;
	union dlm_rh rh;          /* resource handle gets passed back here */
	char		unix_name[MAXRESOURCELEN];
};

/*
 * request that all sites associated with node:pid be freed
 */
struct purgereq {
	short	purgesite;	/* site to purge */
	int	purgepid;	/* pid to purge */
	int 	pid;		/* pid of purger */
	int 	site;		/* site of purger */
	dlm_restype_t type;	/* resource type */
	int 	flags;
};

/* flags for purge request */
#define PURGE_FORCE	1	/* purge ORPHAN locks */
#define PURGE_DEAD	2       /* this client has died */
#define PURGE_ME	4       /* this client is purging itself */

#define PURGE_USRMASK	(PURGE_FORCE | PURGE_DEAD | PURGE_ME)
#define PURGE_REMOTE	8       /* request is being forwarded to purgesite */

/*
 * purge request-response codes
 * ALSO USED as state codes in struct client.cli_state TWICE
 * don't change position or ordering - "next" must be "this" + 1
 * DON'T assign code 0: it means DONE
 * PRSP_BIT is set on all messages to 'client' functionality
 */
#define PREQ_START	0x800	/* from client to initiate purge sequence */
#define PREQ_WANT	0x100	/* request to all nodes that purge is wanted */
#define PRSP_GA		0x900	/* response from all saying go ahead */
#define PREQ_DOIT	0x200	/* request to all saying do purge now */
#define PRSP_PRGD	0xA00	/* response from all saying purged */
#define PREQ_DONE	0x300	/* request to all nodes saying release - done */
#define PREQ_CANCEL	0x400	/* req to all node saying oops */
#define PRSP_FAIL	0xC00	/* resp from any saying 'conflict' */

#define PREQ_MASK	0xF00	/* any bit set? */
#define PREQ_GOOD_MASK	0xB00	/* for increment to 'idle' state */
#define PRSP_BIT	0x800	/* on for any response */
#define PREQ_FIELD	4	/* # bits in field so can dup */

#define PREQ_NEXT_CODE(i) (((i) + 0x100) & PREQ_GOOD_MASK)

/*
 * response flags for clm_reply
 */
#define RESP_UDP	0x1	/* respond by UDP */
#define RESP_AST	0x2	/* respond by AST */
#define RESP_BAST	0x4	/* respond by BAST */
#define RESP_DS_AST	0x8	/* respond by DS_AST */
#define RESP_NULL	0x10	/* no response necessary -jjd- */


struct transaction;
typedef struct transaction clm_trans_t;

/*
 * This structure is contained in a lock record as well as a lock
 * request.  It holds the information necessary to complete a lock
 * request after it has been granted.
 */
struct routeinfo {
	short	respflags;		/* response flags */
	short	rt_master;		/* master id for response */
	void	(*callback)(struct transaction *, struct transaction *);
					/* completion func on secondary */
	union {
		int 		signo;	/* signal to use for notification */
		union dlm_rh	lcl_rh;	/* local resource handle */
	} rt_reuse;
	ast_handle_t asth;		/* for completion ASTs */
	ast_handle_t basth;		/* for blocking ASTs */
	ast_handle_t cbasth;		/* for convert pending blocking ASTs */
};

#define	signo	rt_reuse.signo
#define lcl_rh	rt_reuse.lcl_rh


/*
 * This structure contains information requires for a migration set/get
 * global parameters request.
 */
struct glob_migr_params {
	short			gmp_flags;	/* request flags */
	dlm_globparams_t	gmp_params;	/* migration parameters */
};

/*
 * This structure contains information requires for a migration set/get
 * resource parameters request.
 */
struct res_migr_params {
	char		rmp_resname[MAXNAMLEN];	/* resource name */
	short		rmp_namelen;		/* resource name length */
	short		rmp_type;		/* resource type */
	short		rmp_site;		/* site of requestor */
	short		rmp_flags;		/* request flags */
	struct resource	*rmp_rp;		/* resource cache pointer */
	dlm_resparams_t	rmp_params;		/* migration parameters */
};

/* name information structure for a resource */
struct nameinfo {
	char		name[MAXRESOURCELEN];
	short		namelen;	/* length of name */
};

/*
 * This structure contains information required for an SCN request
 */
struct scninfo {
	int		pid;	/* process ID that requested lock */
	short		site;	/* site # where request was made */	
	short		bit_len;/* bit length of SCN */	
	int 		scn_op;	/* type of SCN operation */
	uint		lockid;	/* lockid */
	struct nameinfo nameinfo;/* resource name information */
	char 		lkvalue[MAXLOCKVAL];
};

union reuse_li {
	struct bounds {
		u_int		lb;		/* lower bound of request */
		u_int		ub;		/* upper bound of request */
	} bounds_unix;
	struct clm_mod {
		u_int		to;
		int		realpid;
	} mod_clm;
} ;

#define	lb	li_reuse.bounds_unix.lb
#define	ub	li_reuse.bounds_unix.ub
#define li_to	li_reuse.mod_clm.to
#define	realpid	li_reuse.mod_clm.realpid

/* 
 *  This structure contains the 8-byte client id.  
 *  The client id can be a XID, or is based on the client's pid or group id
 */
typedef struct clm_client_id {
    int	clid_i[2];
} clm_client_id_t;

/*
 * This is the latest copy of the lockinfo structure updated to contain
 * a transaction identifier.
 *
 * This structure contains lock specific information - most of which
 * originates in the API
 */
struct lockinfo {
	unsigned 	mode:3;		/* grant mode */
	unsigned	reqmode:3;	/* request mode */
	unsigned 	bastmode:3;	/* mode passed back in blocking AST */
	unsigned	site:16;	/* site # where request was made */	
	int 		lk_flags;	/* flags from application */
	int		pid;		/* process ID that requested lock */
	uint		remlockid;	/* remote lockid from secondary */
	uint		lockid;		/* lockid */
	short 		state;		/* internal state flags */
	union dlm_rh	rh;		/* resource handle */
	union reuse_li	li_reuse;
	char 		lkvalue[MAXLOCKVAL];
	struct transaction	*request;/* pointer to create request */
	u_int		seqnum;		/* sequence number on queue */
	dlm_restype_t   type;
	dlm_xid_t	xid;		/* transaction id */
};


/* lock request structure - part of a lock requests transaction */
struct lockreq {
	struct lockinfo lockinfo;
	struct nameinfo nameinfo;
};


/*  Structure to hold all info a reclock needs to keep
    about deadlock detection.  */

struct deadlock {
	dlm_listnode_t   tq;    /*  Links for the waiter-timeout queue.  */
	dlm_listnode_t   cl;    /*  Links for the client/owner list.  */
	/*  Timestamp for Deadlock timeout queue.  */
	struct timeval timestamp;	/* time lock was added to queue */
	struct timeval checkstamp;	/* time last checked for deadlock */
	/*  Deadlock pass stamp to avoid redundant deadlock searches.  */
	short deadlock_stamp;
};

#define DL_TQ_OFS offsetof(struct deadlock, tq)
#define DL_CL_OFS offsetof(struct deadlock, cl)

/* bits in state field of struct lockinfo */
#define 	LOCK_WAITING		0x001
#define 	LOCK_CONVERTING		0x002
#define 	LOCK_ORPHANED		0x004
#define 	LOCK_GONE		0x008
#define 	INFLIGHT_WAITING	0x010
#define 	INFLIGHT_CONVERTING	0x020
#define 	LOCK_LOCAL_ONLY		0x040
#define 	LOCK_AVAILABLE		0x080
#define 	LOCK_STALE		0x100
#define 	LOCK_REFRESHED		0x200
#define 	LOCK_GRANTED		0x400
#define 	LOCK_BASTED		0x800


/*
 * Version 3.1.1
 *
 * this structure contains all information pertain to a lock
 */ 
struct reclock {
	dlm_listnode_t   link;
	struct tq_node	*tq_node;	/* ptr to timeout queue structure */
	struct routeinfo route;
	struct lockinfo	lck;
	struct deadlock dl;
	unsigned char	seg;		/* lock table segment index */
	unsigned char	set;		/* timer set/reset */
};

#define RL_RQ_OFS offsetof(struct reclock, link)
#define RL_TQ_OFS offsetof(struct reclock, dl.tq)
#define RL_CL_OFS offsetof(struct reclock, dl.cl)

#define DEADLOCK_LINK_TO_RECLOCK(n) \
	((struct reclock *) (((char *) n) - RL_TQ_OFS))

#define CLIENT_LINK_TO_RECLOCK(n) \
	((struct reclock *) (((char *) n) - RL_CL_OFS))

/* possible values for clm_direction field */
typedef enum { 
	DLM_REQUEST, 
	DLM_RESPONSE,
	PTI_REQUEST,
	PTI_RESPONSE,
	PTI_ASYNC,		/* asynchronous response */
	PTI_STAT
} ptype_t;	/* packet type */


/* application registration */
struct appreg {
	int pid;
};

#if defined(TIME_STATS)
typedef struct lockstats {
	ushort		slot;
	char		file[15][18];
	int		line[15];
	struct timestruc_t stamp[15];
} timestats_t;	

#define STAMP(req)					\
{							\
	int	i = (req)->stats.slot++;		\
							\
	strcpy((req)->stats.file[i], __FILE__);		\
	(req)->stats.line[i] = __LINE__;		\
	curtime(&((req)->stats.stamp[i]));		\
}

#define PRINTTIME(req)					\
{							\
	int	i, j = (req)->stats.slot;		\
							\
	for (i = 0; i < j; i++)				\
		bsdlog(LOG_NOTICE, "%s [%d]  %d:%d",	\
			(req)->stats.file[i],		\
			(req)->stats.line[i],		\
		(req)->stats.stamp[i].tv_sec%60,	\
		(req)->stats.stamp[i].tv_nsec/1000);	\
}

#else
#define STAMP(req)
#define PRINTTIME(req)
#endif
#ifdef __KERNEL__
extern int k_clm_send(clm_trans_t *usrreq, int len, int clpid);
extern int k_clm_recv(clm_trans_t *usrresp, int len, unsigned int seq, int clpid);
#define fprint_error( format, args... ) \
printk( KERN_ERR format, ## args );
#define fprint_debug( format, args... ) \
printk( KERN_DEBUG format, ## args );
#define print_debug( format, args... ) \
printk( KERN_DEBUG format, ## args );
#else
#define fprint_error( format, args... ) \
fprintf( stderr, format, ## args );
#define fprint_debug( format, args... ) \
fprintf( stderr, format, ## args );
#define print_debug( format, args... ) \
printf( format, ## args );
#endif

/*
 * This data structure is used by the clm_getstats call to request
 * and receive resource statistical information.
 */
struct getstats {
        char              resname[MAXNAMLEN]; /* resource name        */
	short             namelen;            /* resource name length */
	short             type;               /* resource lock type   */
	dlm_statistics_t  statistics;         /* resource statistics  */
};


/*
 * Version 3.1.1
 *
 * this data structure is used throughout the lock manager
 * to carry requests and replies
 */
struct transaction {
  int         clm_client_space;         /* Client in user or kernel space */
  u_long		clm_prog;	/* desired service */
  u_long		clm_vers;	/* service version */
  dlm_stats_t	clm_status;	/* status of request */
  dlm_ops_t	clm_type;	/* request type */
  ptype_t		clm_direction;	/* request or response */
  short		clm_locktype;	/* UNIX or VMS lock */
  unsigned int	clm_sequence;	/* transaction sequence # */
  unsigned int	pti_sequence;	/* transaction sequence # */
  int		clm_authpid;	/* authorized process id of group */
  void            *clm_sender;	/* to detect lost replies */

  /* routing information */
  struct routeinfo clm_route;
  int		clm_pid;	/* process id of client */
  void		*clm_next;	/* next trans or msg */

  union trans_data {
    /* requests */
    struct clm_regreq	_clm_register;	/* DLM_REGISTER */
    struct lockreq		_clm_lockreq;   /* LOCK */
    struct lockinfo		_clm_lockinfo;	/* LOCK,CANCEL,UNLOCK*/
    struct purgereq		_clm_purgelocks;/* DLM_PURGELOCKSPID */
    struct scninfo		_clm_scninfo;	/* DLM_SCN_OP */
    struct res_migr_params	_clm_rmpinfo;	/* DLM_RMIGR_OP */
    struct glob_migr_params	_clm_gmpinfo;	/* DLM_GMIGR_OP */
    struct getstats         _clm_statsinfo; /* DLM_STATS */

    /* responses */
    struct lockstatus	_clm_lockstatus;
    union dlm_rh		_clm_handle;
    struct appreg		_clm_appreg;
    /* holder info will go here for DLM_TEST */
  } clm_data;

#if defined(TIME_STATS)
  timestats_t	stats;
#endif
  /* make sure this stuff is at the end */
  void	*clm_oldtrans;	/* pointer to saved copy of old version */
};

/* union avoiding macros */
#define clm_callback	clm_route.callback
#define clm_asth	clm_route.asth
#define clm_basth	clm_route.basth
#define clm_cbasth	clm_route.cbasth
#define clm_respflags	clm_route.respflags
#define clm_signo	clm_route.signo
#define clm_rt_master	clm_route.rt_master

#define clm_lockreq	clm_data._clm_lockreq
#define clm_register 	clm_data._clm_register
#define clm_lockinfo 	clm_data._clm_lockinfo
#define clm_purgelocks 	clm_data._clm_purgelocks
#define clm_appreg	clm_data._clm_appreg
#define clm_scninfo 	clm_data._clm_scninfo
#define clm_rmpinfo 	clm_data._clm_rmpinfo
#define clm_gmpinfo 	clm_data._clm_gmpinfo
#define clm_statsinfo   clm_data._clm_statsinfo

#define clm_lockstatus	clm_data._clm_lockstatus
#define clm_handle	clm_data._clm_handle

/* Client macros */
/*
 * determine if two reclocks are held by the same client
 */
#define same_proc(x, y) ((x)->pid == (y)->pid && \
			 (x)->site == (y)->site)

/* set the client id using the pid or groupid */
#define PROC_CLID_VAL	0xc1a
#define IS_PROC(clid)	((clid).clid_i[0] == PROC_CLID_VAL)
#define	CLID2PID(clid)	((IS_PROC(clid)) ? (clid).clid_i[1] : 0)
#define set_client_id(x, y)  (x).clid_i[0] = 0xc1a; (x).clid_i[1] = (y);
#define set_client_idp(x, y)  (x)->clid_i[0] = 0xc1a; (x)->clid_i[1] = (y);


/* print out clid for debugging purposes */
#define db_clid(x)    (x).clid_i[0], (x).clid_i[1]
#define db_clidp(x)    (x)->clid_i[0], (x)->clid_i[1]

/*
 *	Group structures use to maintain lock groups.
 */
typedef struct members {
	int		gm_pid;		/* pid of proc in group */
	int		gm_flags;	/* member control flags */
	struct members	*gm_next;	/* pointer to next member */
} grp_mem_t;

typedef struct clmgroup {
	int		grp_id;		/* pid of client */
	int		grp_flags;	/* control flags */
	grp_mem_t	*grp_mbrs;	/* list of group members */
} clm_group_t;

typedef struct grpify {
	int		gi_gid;
	int		gi_site;
	int		gi_pid;
} clm_grpify_t;

#define	CLMG_GROUPIFY	0x1

/*
 *  NACK Message Types
 */
enum nack_message_types {
        NACK_SEND /* Sending NACK message */
};


struct locallock_broadcast;
typedef struct locallock_broadcast locallock_broadcast_t;


/*
 * Client types: client can be a process, group or transaction 
 */
typedef enum
{
	CLM_CLI_PROCESS,
	CLM_CLI_GROUP,
	CLM_CLI_TRANSACTION,
	CLM_CLI_PURGE
} clm_client_type_t;

struct noderesp
{
    unsigned short nr_node ;
    unsigned short nr_resp ;
} ;

/*  
  Client structure used to keep an ordered list of clients,
  by client id and site, with a doubly linked list of all reclock's
  owned by that client.

  Keeps track of state (client and servers) for purge processing.
  Holds info for purge deferred for migration to complete.
  Present on master and client nodes for all locks
	Present on all nodes during purge
 */

struct clm_client;
typedef struct clm_client clm_client_t;


#ifdef __KERNEL__

struct clm_client
{
  clm_client_id_t   cli_id;             /* client id */
  int               cli_site;           /* site of client */
  int               cli_groupid;        /* group id of client if any */
  clm_client_type_t cli_type;           /* client type: PROCESS, 
                                             GROUP or TRANSACTION */
  unsigned int      cli_sequence;       /* client sequence */
  dlm_list_t        sync_seq;           /* seq # of lock request */
  dlm_list_t        cli_queue;          /* owned locks list */
  dlm_list_t        cli_ast_pnd;        /* list of pending ASTs */

  /* Used for syscall interface (see cllockd/clm_cti.c).  -jjd- */
  struct transaction *cli_reply;

  wait_queue_head_t cli_reply_event;
  int               cli_migrcount ;     /* # res w/locks migrating */
  unsigned int      cli_state ;         /* state code & flags */
  /* flags for nodes replying */
  struct noderesp   cli_resp[MAXNODES] ;

#ifdef CONFIG_PROC_FS
  struct proc_dir_entry * proc_entry;
#endif /* CONFIG_PROC_FS */
};


#define CLI_NEED_PURGE 0x1000	/* purge deferred for migr */
#define CLI_NEED_DEAD 0x2000	/* purge dead needed - start new cycle */

				/* mask for client (purge requestor) state */
#define CLI_CL_STATE	PREQ_MASK
				/* mask for server (purge processor) state */
#define CLI_SV_STATE	(CLI_CL_STATE >> PREQ_FIELD)

/* CLI_CODE == 0 -> no purge requested or in progress */

#define CLI_CODE	(CLI_CL_STATE | CLI_SV_STATE)

/*
 * macro to set server state in struct client from a PREQ code 'f'
 */
#define CLI_SV_SET(cp, f) do \
{\
    (cp)->cli_state &= ~CLI_SV_STATE ; \
    (cp)->cli_state |= ((f) & PREQ_MASK & ~PRSP_BIT) >> PREQ_FIELD ; \
} while (0)

#define CLI_SV_GET(cp) ((cp->cli_state & CLI_SV_STATE) << PREQ_FIELD)
#define CLI_CL_GET(cp) (cp->cli_state & CLI_CL_STATE)

/*
 * macro to set purge client state in struct client from a PREQ code 'f'
 */
#define CLI_CL_SET(cp, f) do \
{\
    cp->cli_state &= ~CLI_CL_STATE ; \
    cp->cli_state |= (f) & PREQ_MASK ; \
} while (0)

/*
 * client state codes are the same as the corresponding requests
 * see definition at 'struct purgereq'
 */


/* Indices for mutex locks.  -jjd- */
#define	LK_ENTRY	0

#ifdef NOTDEF
/* Event mask used when waiting for reply from primary.  -jjd- */
#define REPLY_MASK	0x0100
#endif

#endif /* __KERNEL__ */

#define	MAX_SCN_SIZE	(MAXLOCKVAL*8)

/*
 * Map structure for scn mapping over a lock value block 
 */
typedef struct scn {
	unsigned int	base;
	unsigned int	wrap1;
	unsigned int	wrap2;
	unsigned int	wrap3;
} scn_t;

/* directory information structure */
typedef struct rl_info {
	dlm_listnode_t  link;
	void		*rl_name;	/* name field of the resource */
	ushort		rl_namelen;	/* length of the name value */
	ushort		rl_flags;	/* RLDB flags */
	ushort		rl_node;	/* node where resource is mastered */
	unsigned char rl_type;	/* type of resource */
} rl_info_t;

/*
 * directory information update structure
 * The rl_flags field was added for 3.1.1, it was placed at end of structure so
 * ver 3.1 can pass dir update info correctly to newer versions. Newer versions
 * must check where dir update request originated before writing to rl_flags.
 */
typedef struct rl_update {
	char		rl_name[MAXRESOURCELEN];/* name field of the resource */
	ushort		rl_namelen;	/* length of the name value */
	ushort		rl_node;	/* node where resource is mastered */
	unsigned char rl_type;	/* type of resource */
	ushort		rl_flags;	/* RLDB flags, added for 3.1.1 */
	struct resource	*rl_resptr;	/* pointer to update resource */
} rl_update_t;

/* RLDB flags */
#define	RLINFO_DIRECTORY	0x01	/* set for a directory entry */
#define	RLINFO_FROZEN		0x02	/* entry is frozen - don't use */
#define	RLINFO_TOUCHED		0x04	/* used for LRU reclaim algorithm */
#define RLINFO_LOCAL            0x08    /* set for local lock in reconfig */

#define	RLENUM_ALLNODES		0
#define	RLENUM_SPECNODES	1

/* update_directory flags */
#define	DIR_DELETE		0

/* Ports used */
#define	PORT_DIRECT	1	/* Directory server port for lock requests */
#define	PORT_MSTR	2	/* Master port for lock reqs	*/
#define	PORT_RESPONSE	3	/* Comm port for master resp	*/
#define	PORT_DIRUPDT	4	/* Dir server port for updates	*/
#define PORT_MIGR	5	/* Port for all migr traffic	*/
#define PORT_MSTR_CR	6	/* Port for lock req w/ master creat */
#define PORT_RECOVERY	7	/* Port for all recovery messages */
#define	PORT_DEADLOCK	8	/* Port for deadlock detection	*/
#define	PORT_BOUNCE	9	/* Rubber port			*/
#define	PORT_GROUP	10	/* Group control port		*/
#define PORT_NACK       11	/* Port used to send NACKs      */
#define PORT_SYNC	12	/* 4.2-style inband sync	*/
#define PORT_SYNCRET	13	/* 4.2 inband sync inband ack */

/* Priorities used */
#define BASEPRIO	10	/* Base priority for all messages	*/
#define CONTPRI		100	/* Base priority for continued messages	*/
#define MIGAPRI		50	/* Priority for migration acks		*/
#define DIRRPRI		CONTPRI	/* Priority for directory redirects	*/
#define RESRPRI		150	/* Priority for resource rebuilds	*/

#define	DLM_RES_CREATE	0x1	/* bit set for master fn() if res creation OK */
#define	DLM_TUNE_SET	0x1	/* bit set for migration param set operation */

/* bit values to track message states in cm_private */
#define	MSG_CONFIRMED	0x1	/* send callback has been run on this msg   */
#define	MSG_MATCHED	0x2	/* response for this message received       */
#define	MSG_BOUNCE	0x4	/* set in message to prevent migration loop */
#define	MSG_BNCE_DONE	0x8	/* set in message to cause it to be dropped */

struct clm_syncmsg
{
    int sender ;
    int event ;
    int sync_id ;
} ;

#ifndef HAVING_NLS_ON_LINUX
/*
 * These are message identifiers that are used in messages all
 * over the place.  Under a perfect system, they would be the
 * translated message catalogs, and these codes would be the
 * keys - through catopen/catgets to retrieve the message.
 * For now though, we merely want these to translate to a
 * value, any value, and let the other definitions take care
 * of things.
 */
/* $ Message for cllockd.c */
#define LOAD_KERNEL 1 /*	"Loaded kernel extension.\n" */
#define LOAD_STARTUP 1 /*	"Loaded startup code.\n" */
#define INTERNAL_ERR 1 /*	"Internal error (could not find start_func).\n" */
/* $ */
/* $ Do not translate following messages until you find a comment */
/* "End of non-transable messages". */
/* $  */
/* $ Message for clm_cti.c */
#define RES_QUEUED 1 /*	"Response queued without signal!!!\n" */
#define REMOT_FUNC_FAIL 1 /*	"Remote function failed!!!\n" */
#define CLM_VOID_INSEND 1 /*	"CLM_VOID in sendudp.\n" */
/* $ */
/* $ Message for kernel.c */
#define EXIT_WITH_OP 1 /*	"Exiting with %ld.\n" */
#define UNLOAD_ING 1 /*	"Unloading %ld.\n" */
#define ABORT_ING 1 /*	"Aborting.\n" */
#define GETSERV_BY_NAME 1 /*	"Getservbyname(%s).\n" */
#define DIED_HOLDING 1 /*	"%ld died holding a lock!!\n" */
#define NESTED_LOCK 1 /*	"Nested lock, id = %ld, process = %ld.\n" */
#define LOCK_FAILED 1 /*	"LOCK_FAIL?\n" */
#define LOCK_SIGNAL 1 /*	"LOCK_SIG?\n" */
#define UNKNOWN_LOCK_STUS 1 /* "Unknown lock status %ld.\n" */
#define WAIT_INTRUPT 1 /*	"Wait interrupted.\n" */
#define UNKNOWN_WAIT_STUS 1 /* "Unknown wait status 0x%x.\n" */
/* $ "End of non-transable messages". */
/* $ */
/* $ Messages from kernel.c to translate */
#define HEAP_DESC_ALLOC_FAIL 1 /* "Heap_desc allocation failed.\n" */
#define HEAP_SPEC_ALLOC_FAIL 1 /* "Heap space allocation failed.\n" */
#define TOO_MANY_BLOK_ALLOC 1 /* "Too many big blocks allocated.\n" */
#define HEAP_ALLOC_FAIL 1 /* "Heap allocation failed.\n" */
#define FREE_BYTES 1 /*	"Freeing %ld bytes at 0x%x.\n" */
#define FREE_ALIEN_BLOK 1 /*	"Tried to free alien block at 0x%x.\n" */
#define INVALID_STAT 1 /*	"Invalid stat %ld.\n" */
/* $ */
/* $ Do not translate following messages until you find a comment */
/* $ "End of non-transable messages". */
/* $ */
/* $ Message for clm_api.c */
#define DLM_REG_FAIL 1 /*	"dlm_register: failed call to mlm.\n" */
#define DLM_REG_TOO_MANY_RES 1 /* "dlm_register: too many resources.\n" */
#define DLM_REG_BAD_RES 1 /*	"dlm_register: bad resource handle.\n" */
#define DLM_REG_INFO 1 /*	"dlm_register: name %s type %ld handle 0x%lx\n" */
/* $ */
/* $ Message for clm_main.c */
#define CANT_EST_DEADMAN_SOCK 1 /* "Can't establish deadman socket to clstrmgr.\n" */
#define CANT_INT_AST_SUBSYS 1 /* "Can't initialize AST subsystem.\n" */
#define CANT_ALLOC_RES_TAB 1 /* "Unable to allocate resource tables of size %ld.\n" */
#define CANT_REGISTER_UDP 1 /* "%s: unable to register (%s, %s, udp).\n" */
#define CANT_REGISTER_TCP 1 /* "%s: unable to register (%s, %s, tcp).\n" */
#define BYE_BYE	 1 /*	"Bye!!\n" */
#define LOCK_DEADMAN_CANT_RESTRT 1 /* "Lock daemon cannot be restarted.\n" */
/* $ */
/* $ Message for clm_io.c */
#define SIGNAL_SEND 1 /*	"Signal %ld in send.\n" */
#define ERR_ON_SEND 1 /*	"Error %ld on send.\n" */
#define SIGNAL_RECV 1 /*	"Signal %ld in recv.\n" */
/* $ */
/* $ "End of non-transable messages". */
/* $ Message for loadit.c */
#define CANT_OPEN 1 /*	"Can't open: \n" */
#define LOG_TO_FILE 1 /*	"logging to file %s.\n" */
#define L_CANT_EXCEED 1 /*	"-L can't exceed %ld or be less than 1\n" */
#define R_CANT_EXCEED 1 /*	"-R can't exceed %ld or be less than 1\n" */
#define P_CANT_EXCEED 1 /*	"-P can't exceed %ld or be less than 0\n" */
#define H_CANT_EXCEED 1 /*	"-H can't exceed %ld or be less than 1\n" */
#define D_RANGE_ERROR 1 /*	"-D can't exceed %5.3f or be less than %5.3f\n" */
#define D_RANGE_DEF 1 /*	"Using default rate of %5.3f\n" */

#define CANT_FIND_CLMLKM 1 /* "Could not find clm_lkm port.\n" */
#define CANT_FIND_CLLOCKD 1 /* "Could not find cllockd port.\n" */
#define CANT_FIND_CLMPTS 1 /* "Could not find clm_pts port.\n" */
#define CANT_FIND_CLINFODEADMAN 1 /* "Could not find clinfo_deadman port.\n" */
#define DEBUG_ENABLE 1 /*	"%s debugging enabled.\n" */
#define BAD_OPTION 1 /*	"bad option flag '%s'\n" */
#define DEBUG_OPTIONS 1 /*	"Debug Options are:\n" */
/* $ */
/* $ Message for stopit.c */
#define INTERNAL_ERROR 1 /*	"Internal error.\n" */
#define KILLING_PID 1 /*	"Killing %ld\n" */
#define KERNEL_NOT_LOAD 1 /* "The kernel extension has not been loaded.\n" */
#define KERNEL_ALD_UNLOAD 1 /* "The kernel extension has already been unloaded.\n" */
#define UNLOAD_KERNEL 1 /*	"Unloaded kernel extension.\n" */
/* $ */
/* $ Do not translate following messages until you find a comment */
/* $ "End of non-transable messages". */
/* $ Message for svc.c */
#define GETREQ_AUTH_FAIL 1 /* "svc_getreq: auth failed.\n" */
#define GETREQ_BAD_VER 1 /*	"svc_getreq: bad prog or version.\n" */
#define SVC_XPRT_DIED 1 /*	"svc: xprt_died.\n" */
#define SVC_CANT_MBUF 1 /*	"svc: count %ld mbuf %x .\n" */
#define SVC_REQ_COUNT 1 /*	"svc after req: count %ld mbuf %x .\n" */

/* $ Messages for clm_alloc.c */
#define NEG_LOCKID 1 /*	"get_le: negative lockid %ld" */
#define NO_MOR_LKS 1 /*	"get_le: NO MORE LOCKS" */
#define CANT_MALLOC 1 /*	"malloc: can't malloc locks" */
#define INDX_OUT_RANGE 1 /*	"find_reclock: index %ld out of range\n" */
#define SEG_OUT_RANGE 1 /*	"find_reclock: segment %ld out of range %ld for type 0x%x\n" */
#define WRONG_RELOCK 1 /*	"find_reclock: got wrong reclock" */

/* $ Messages for clm_batch.c */
#define MALLOC_FAILED 1 /*	"clm_alloc_trans: malloc failed" */
#define BUFFER_UNDERFLOW 1 /*	"clm_alloc_trans: buffer underflow" */
#define FREE_BUFFER 1 /*	"clm_free_trans: freeing foreign buffer. Address = 0x%x" */
#define INCOMP_SEND 1 /*	"Incomplete send in pti_flush_responses()" */
#define SENT_X_Y 1 /*	"sent %ld of %ld from 0x%x" */
#define ERROR_DETECTED 1 /*	"Error detected in clm_response_complete\n" */
#define MALLOC_ERR 1 /*	"malloc error in clm_queue_response\n" */
#define QUE_OVERFLOW 1 /*	"inflight queue exceeds 500 entries" */
#define LOST_REPLY 1 /*	"ERROR: lost a reply from the primary. Aborting" */
#define XMT_FAILED 1 /*	"Failed to retransmit inflight queue" */
#define TRAN_RESP_FAIL 1 /*	"Error responding to resend transaction" */

/* $ Messages for clm_client.c */
#define NUM_TRSH_REPLY 1 /*	"trashed %ld replies" */
#define NO_CLIENT 1 /*	"removeclient: no such client: %ld" */
#define ADD_CLIENT_FAIL 1 /*	"add_client_reclock: couldn't add client." */

/* $ Message for clm_clmlock.c */
#define NULL_RET 1 /*	"clm_lock:clm_direct() returned NULL" */
#define CANT_CONVERT 1 /*	"begin_lock_clm: can't get resource for converting lock rh=0x%x\n" */
#define CANT_RELOCK 1 /*	"cont_lock_clm: ERROR: can't find reclock for blocked lock id=0x%x" */
#define NULL_HANDLE 1 /*	"cont_lock_clm: NULL resource handle for 0x%x" */
#define NULL_HANDLE_CONVERT 1 /*	"clm_convert: NULL resource handle for 0x%x" */
#define CANT_GET_RES 1 /*	"cont_lock_clm: can't get resource for converting lock rh=0x%x\n" */
#define CANT_GET_RES_LOCK 1 /*	"cont_lock_clm: can't get resource for new lock rh=0x%x" */
#define NULL_RET_CONVERT 1 /*	"clm_convert:clm_direct() returned NULL" */
#define NULL_RET_MASTER	 1 /*	"master_unlock_clm:clm_direct() returned NULL" */
#define NO_LOCK_ON_MSTR 1 /*	"clm_convert:  lock but no resource on non-master" */

/* $ Message for clm_clmunlock.c */

#define NULL_LOCK_HANDLE 1 /*	"pre_proc_unlock_clm: NULL resource handle for 0x%x" */
#define NO_HANDLE 1 /*	"pre_proc_unlock_clm: can't find handle for 0x%x" */
#define RET_NULL_DIRECT 1 /*	"master_unlock_clm:clm_direct() returned NULL" */
#define NO_RES_HANDL 1 /*	"master_unlock_clm: NULL resource handle for 0x%x" */
#define UNLOCK_NO_HANDLE 1 /*	"master_unlock_clm: can't find handle for 0x%x" */
#define NULL_RETURN 1 /*	"master_unlock_clm:clm_direct() returned NULL" */
#define RELOCK_ERR 1 /*	"local_unlock_clm: can't find reclock for 0x%x" */
#define LOCAL_NULL_RES 1 /*	"local_unlock_clm: NULL resource handle for 0x%x" */
#define LOCAL_UNLK_ERR 1 /*	"local_unlock_clm: can't find handle for 0x%x" */

/* $ Message for clm_cti.c */
#define BUFFER_ERR 1 /*	"clm_send: can't get receive buffer" */
#define COPY_FAILED 1 /*	"Copyin failed" */
#define CLIENT_NOT_FOUND 1 /*	"clm_recv could not find client %ld" */
#define COPY_OUT_FAILED 1 /*	"Copyout failed" */
#define ILLEGAL_CLIENT 1 /*	"clm_process: Unrecognized client %ld" */
#define BAD_REQUEST 1 /*	"clm_request: bad request type %ld" */
#define NULL_RESP 1 /*	"clm_reply: response and request NULL" */
#define CLIENT_NOT_FOUND2 1 /*	"sendudp: could not find client %ld" */
#define BUFFER_SHORTAGE	 1 /*	"sendudp: could not allocate buffer" */
#define NOTIFY_NULL_HANDLE 1 /*	"notify: NULL resource handle" */
#define BAD_RESPTR 1 /*	"notify: bad resptr, rh=0x%x\n" */
#define BAD_PARAM 1 /*	"notify: bad HOW parameter" */
#define AST_FAILED 1 /*	"sendast: send failed AST" */
#define BAST_FAILED 1 /*	"sendast: send failed for BAST" */
#define NO_VALID_HANDLE 1 /*	"send_ast: no valid handle" */

/* $ Messages for clm_deadlock.c */
#define ERR_MALLOC 1 /*	"Malloc error in clmdd_startstart()" */
#define ERR_MALLOC1 1 /*	"Malloc error in clmdd_dostart()" */
#define CANT_LOOKUP_RES 1 /*	"clmdd_pushr: can't look up resource" */
#define ERR_MALLOC2 1 /*	"Malloc error in clmdd_pushr()" */
#define NO_RES_HNDL 1 /*	"clmdd_analyze: can't get resource handle" */
#define ERR_MALLOC3 1 /*	"Malloc error in clmdd_processddx()" */
#define CANT_LOOKUP_RES2 1 /*	"clmdd_massage: can't look up resource" */
#define ERR_MALLOC4 1 /*	"Malloc error in ddx_alloc()" */
#define ERR_MALLOC5 1 /*	"Malloc error in ddx_expand()" */
#define ERR_MALLOC6 1 /*	"Malloc error in clmdd_encode()" */
#define PURG_DEADLOCK 1 /*	"purge_deadlock: bad resptr, rh=0x%x\n" */

/* $ Message for clm_debug.c */
#define LOG_ERROR 1 /*	"%s: Error %ld" */

/* $ Message for clm_direct.c */
#define UNKNOWN_DIR 1 /*	"Unknown directory request type in dir_proc_request" */
#define DROP_REQUEST_CONV 1 /*	"Master convert request for %ld:%s not\n found by directory server.  Dropping request." */
#define DROP_REQUEST_UNLOCK 1 /*	"Master unlock request for %ld:%s not\n found by directory server.  Dropping request." */
#define DROP_REQUEST_PURGE 1 /*	"Master purge request for %ld:%s not\n found by directory server.  Dropping request." */
#define INVALID_TRAN 1 /*	"Invalid trans type for directory create" */
#define RL_MODIFY_FAIL 1 /*	"dir_proc_request: rl_modify failed" */
#define RL_CREATE_FAIL 1 /*	"dir_proc_request: rl_create failed" */
#define ERR_MALLOC7 1 /*	"Malloc error in update_directory()" */
#define DIR_NODE_ERR 1 /*	"Unable to determine directory node" */

/* $ Messages for clm_io.c */
#define BAD_PROCID 1 /*	"clm_write: bad kproc_pid = %ld" */
#define BAD_PROCID_RD 1 /*	"clm_read: bad kproc_pid = %ld" */
#define WRT_EV_FAULT 1 /*	"writev emulation faulted (%ld)\n" */
#define SEND_FAILED 1 /*	"send failed for 0x%x" */
#define BATCHSIZE_ERR 1 /*	"clm_iov: batchsize %ld greater than CLM_BATCHSIZE %ld" */
#define BUFFER_ERR2 1 /*	"clm_iov: nbufs(%ld) != number of buffers(%ld)" */

/* $ Messages for clm_libr.c */
#define NO_CLSTR_MGR 1 /*	"Error, no cluster manager running" */
#define PTI_CALL_ERR 1 /*	"Error detected in pti_call_complete\n" */
#define PTI_MALLOC_FAIL 1 /*	"malloc failed in pti_call()\n" */
#define CLM_DIRECT_NULL 1 /*	"clm_direct() returned NULL" */
#define ERR_MALLOC8 1 /*	"malloc failed in pti_call_reg()\n" */
#define ERR_MALLOC9 1 /*	"malloc failed in pti_call_unix()\n" */

/* $ Message for clm_lock.c */
#define DUP_RELOCK 1 /*	"add_queue: duplicate reclock" */
#define CORRUPT_Q 1 /*	"add_queue: Corrupt Queue" */
#define CORRUPT_REM_Q 1 /*	"remove_queue:Corrupt Queue" */
#define ERR_MALLOC10 1 /*	"malloc failed in pti_purge()\n" */
#define MALFORMED_Q 1 /*	"Malformed QUEUE. ABORTING" */

/* $ Message for clm_lcokid.c */
#define FREEMAP_ERR 1 /*	"freemap: not allocated" */
#define REMOTE_FREEMAP 1 /*	"freemap: can't find map for remoteid id %ld" */
#define HASHTABLE_ERR 1 /*	"addmap: can't malloc hash table" */
#define HASHED_REMOTE_ID 1 /*	"ERROR: remoteid 0x%x already hashed" */
#define MAPID_ERR 1 /*	"mapid: not allocated" */
#define REMOTE_FREEMAP2 1 /*	"mapid: can't find map for remoteid 0x%x" */

/* $ Messages for clm_main.c */
#define PTI_REQ_ERR 1 /*	"Error returned from pti_proc_request" */
#define ILLEGAL_REQ 1 /*	"pti_prog_p: illegal request type %ld seq 0x%x" */
#define USE_XMALLOC 1 /*	"Had to use xmalloc for proc_notify\n" */
#define NULL_PTR 1 /*	"NULL pointer passed to free_dead()\n" */
#define CORRUPT_DPTR 1 /*	"free_dead: found dptr from outside pool 0x%x\n" */
#define FAIL_TO_ALLOCATE 1 /*	"Allocation failed in purge_list" */
#define DEAD_PID 1 /*	"%s: can't allocate dead pid structs" */
#define CANT_DETER_INFO 1 /*	"Unable to determine local site info" */
#define CANT_INIT_COMM_LYR 1 /*	"cllockd: unable to init comm layer" */
#define CANT_INIT_PTI_PORT 1 /*	"cllockd: unable to init PTI request port" */
#define CANT_INIT_DIR_PORT 1 /*	"cllockd: unable to init directory port" */
#define CANT_INIT_RES_PORT 1 /*	"cllockd: unable to init PTI response port" */
#define CANT_INIT_RLDB	 1 /*	"cllockd: unable to init RLDB" */
#define CANT_INIT_CLMM	 1 /*	"cllockd: unable to init CLMM" */
#define CANT_INIT_DEADLOCK_DET 1 /*	"cllockd: unable to init deadlock detection" */
#define EXIT_ON_SIGNAL	 1 /*	"Exiting on signal %ld" */
#define SERVER_LOOP_ERR	 1 /*	"kern_main: server loop error" */
#define CM_DIED		 1 /*	"Cluster Manager has died, exiting." */
#define BAD_DIRECTION_SPEC 1 /*	"bad direction specifier in PTI request" */
#define STRAY_RESPONCE	 1 /*	"Got stray response!" */
#define MATCH_REQ_INTERR 1 /*	"match_request interrupted" */

/* $ Messages for clm_prot.c */
#define SND_RES_TBL_FAIL 1 /*	"send_resource_table failed.  Restart cllockd" */

/* $ Messages for clm_resource.c */
#define RES_TABLE_OVERFLOW 1 /*	"%s: resource table overflow" */
#define ALLOCATED_SEGMENT 1 /*	"%s: segment %ld already allocated" */
#define ERR_MALLOC11	 1 /*	"%s: malloc error expanding restab" */
#define REHASH_ERR	 1 /*	"find_reshash: can't allocate reshash\n" */
#define FOUND_EXISTING_HASH 1 /*	"find_reshash: found existing hash for new resource" */
#define CANT_FIND_DELETE 1 /*	"find_reshash: can't find/delete %s" */
#define BAD_LEN_RES	 1 /*	"allocate_resource: bad length (%ld)" */
#define RES_SEG_FULL	 1 /*	"%s: all resource segments full" */
#define RES_SEG_ALLOCATE_FAIL 1 /*	"%s: failed to allocate res segment" */
#define RES_SEG_FIND_ERR 1 /* 	"%s: failed to find res segment" */
#define NO_MT_RES_SLOT	 1 /*	"%s: couldn't find empty resource slot" */
#define ERR_MALLOC12	 1 /*	"%s: malloc error getting resource name" */
#define NON_MT_LOCKLIST	 1 /*	"%s: non-empty lock list for %s\n" */
#define NON_MT_RESREF_LIST 1 /*	"%s: non-empty resref list for %s\n" */
#define NULL_RETURNED	 1 /*	"clm_lock:clm_direct() returned NULL" */
#define PTI_CALL_FAILED	 1 /*	"begin_register: failed call to PTI" */
#define TOO_MANY_RESOURCES 1 /*	"cont_remote_register: too many resources" */
#define RESOURCE_FAILURE 1 /*	"cont_remote_register: resource failure" */
#define XMIT_FAILED	 1 /*	"Failed to xmit resource packet to primary e=%ld" */
#define OUT_OF_RANGE_Q	 1 /*	"%s: Queue type out of range.\n" */
#define XMIT_LOCK_PKT_FAIL 1 /*	"Failed to xmit 2 lock packet to primary e=%ld" */
#define XMIT_LOCK_PKT_FAIL2 1 /*	"Failed to xmit 1 lock packet to primary e=%ld" */
#define NO_RESOURCE_SLOT 1 /*	"receive_resource: couldn't allocate resource slot" */
#define INVALID_RES_HANDLE 1 /*	"receive_lock: invalid resource handle" */
#define CANT_RELOCK2	 1 /*	"receive_lock: unable to alloc reclock" */
#define MAP_REMOTE_LOCKID 1 /*	"receive_lock: map remote lockid" */

/* $ Messages for clm_rldb.c */
#define ERR_MALLOC13	 1 /*	"Malloc failed in rl_expand_freelist" */
#define NO_MEM_FOR_RLDB	 1 /*	"rl_init: unable to allocate memory for RLDB" */
#define BAD_ARGS	 1 /*	"Bad arg passed to rl_enum\n" */

/* $ Messages for clm_scn.c */
#define NULL_RES_HANDLE	 1 /*	"begin_scn_op: NULL resource handle for 0x%x" */
#define CANT_GET_HANDLE	 1 /*	"begin_scn_op: can't find handle for 0x%x" */
#define NULL_RES_HANDLE2 1 /*	"local_scn_op: NULL resource handle for 0x%x" */
#define CANT_GET_HANDLE2 1 /*	"local_scn_op: can't find handle for 0x%x" */

/* $ Messages for clm_timeout.c */
#define BAD_RESPTR2	 1 /*	"send_timeouts: bad resptr, rh=0x%x\n" */
#define CANT_ALLOCATE_TRANS 1 /*	"send_timeouts: could not allocate transaction" */

/* $ Messages for clm_tune.c */
#define ERROR_IN_PROC	 1 /*	"Error detected in send_glob_params_complete" */
#define ERR_MALLOC14	 1 /*	"%s: malloc failed" */
#define CCCP_SEND_RETURNED 1 /*	"%s: cccp_send returned %ld\n" */
#define BAD_RETURN	 1 /*	"Bad return from clmm_ctrl\n" */

/* $ Messages for clm_unixlock.c */
#define CANT_DETERMINE_DIR_NODE 1 /*	"Can't determine directory node" */
#define CANT_ALLOCATE_LK_ENTRY 1 /*	"master_lock_unix: can't allocate lock entry" */
#define CANT_FIND_RESOURCE 1 /*	"Can't find resource in local_lock_unix" */
#define CANT_ALLOCATE_LK_ENTRY2 1 /*	"local_lock_unix: can't allocate lock entry" */
#define CANT_GET_LOCK_ENTRY 1 /*	"%s: can't get lock entry" */
#define IMPOSSIBLE	 1 /*	"%s: impossible!" */
#define BAD_RESPTR_SRCH_LK	 1 /* "search_lock: bad resptr, rh=0x%x\n" */
#define CANT_ALLOCATE_LK_ENTRY3 1 /*	"insert_le: can't allocate lock entry" */
/* $ "End of non-translatable messages". */
#endif  /* !HAVING_NLS_ON_LINUX */

void clm_sleep (int secs);

static __inline__ void api_sleep(int secs)
{
#ifdef __KERNEL__
    clm_sleep(secs);
#else
    sleep(secs);
#endif /* __KERNEL__*/
}
#endif /* !CLMSTRUCTS_H */
