/*
 * Core modules for the IBM OSS Distributed Lock Manager
 *
 * Copyright (C) 2001, International Business Machines
 *
 * Authors: Peter R. Badovinatz, Ian D. Romanick
 * Contacts: wombat@us.ibm.com, idr@us.ibm.com
 * http://oss.software.ibm.com/developerworks/opensource/linux/projects/dlm
 *
 * This program is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY;  without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 * the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program;  if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
#ident "$Id: cltrc.h,v 1.3 2001/09/05 18:25:38 idr Exp $"

/*
 *   COMPONENT_NAME: INCCLUSTER
 *
 *   FUNCTIONS: defined
 */

#if !defined(CLTRC_H)
#define CLTRC_H

#ifdef __linux__
/*
 * Linux - has none of this trace support, so, just set macros
 * that null it out for now.  Later, would be useful to replace
 * these with functional tracing, as debugging will be a bit of
 * a pain until then...
 */
extern void no_trace(int hook);
#define TRCHKGT(hook, who, a1, a2, a3, a4) no_trace(hook)
#define TRCHKL0T(hook) no_trace(hook)
#define TRCHKL1T(hook, a1) no_trace(hook)
#define TRCHKL2T(hook, a1, a2) no_trace(hook)
#define TRCHKL3T(hook, a1, a2, a3) no_trace(hook)
#define TRCHKL4T(hook, a1, a2, a3, a4) no_trace(hook)
#define TRCHKL5T(hook, a1, a2, a3, a4, a5) no_trace(hook)
#define TRCHKL6T(hook, a1, a2, a3, a4, a5, a6) no_trace(hook)

#define ATT_UNUSED __attribute__ ((__unused__))

inline static void cltrc(int a1 ATT_UNUSED, int a2 ATT_UNUSED,
			 int a3 ATT_UNUSED, int a4 ATT_UNUSED,
                         void * a5 ATT_UNUSED,
                         void * a6 ATT_UNUSED,
                         void * a7 ATT_UNUSED,
                         void * a8 ATT_UNUSED)
{
  return;
}

#undef ATT_UNUSED

#endif /* __linux__ */

/* 
 * We keep these defined, so that we can optionally (with a
 * debug flag) print something with these trace options
 * going.
 */

/* App level */
#define L_KERNL 0	/* kernel */
#define L_USR   1	/* user */

/* SRC to Appl returned tasks */
#define NULL_MSG   	0	/* Null message */
#define STOP_NORMAL   	10	/* Normal stop */
#define STOP_FORCED   	11	/* Forced stop */
#define STOP_CANCEL   	12	/* Cancel stop */
#define REFRESH_SRC   	22	/* refresh */
#define TRACE_ON_SHORT 	40 	/* Start short trace request */
#define TRACE_ON_LONG  	41 	/* Start long trace request */
#define TRACE_OFF      	42 	/* Stop trace request */
#define DM_CONT_RECONFIG  50    /* Continue reconfiguration */

/* 
 * Debug levels sent to the cluster manager in 'parm1' as part of a 
 * DM_DEBUG action.  The levels really go from 0 - 9, but we start at 60
 * here for uniqueness.
 */

#define DM_DEBUG_LEVEL_BASE 60

#define DM_DEBUG_LEVEL_0  (0 + DM_DEBUG_LEVEL_BASE)
#define DM_DEBUG_LEVEL_1  (1 + DM_DEBUG_LEVEL_BASE)
#define DM_DEBUG_LEVEL_2  (2 + DM_DEBUG_LEVEL_BASE)
#define DM_DEBUG_LEVEL_3  (3 + DM_DEBUG_LEVEL_BASE)
#define DM_DEBUG_LEVEL_4  (4 + DM_DEBUG_LEVEL_BASE)
#define DM_DEBUG_LEVEL_5  (5 + DM_DEBUG_LEVEL_BASE)
#define DM_DEBUG_LEVEL_6  (6 + DM_DEBUG_LEVEL_BASE)
#define DM_DEBUG_LEVEL_7  (7 + DM_DEBUG_LEVEL_BASE)
#define DM_DEBUG_LEVEL_8  (8 + DM_DEBUG_LEVEL_BASE)
#define DM_DEBUG_LEVEL_9  (9 + DM_DEBUG_LEVEL_BASE)

/*
 * New SRC params for DM_DEBUG.  This is how we turn debugging on and off
 * while the cluster manager is running.
 */

#define DM_DEBUG_ON     51      /* Turn debugging on.  'parm1' is the debug */
                                /* level */
#define DM_DEBUG_OFF    52      /* Turn debugging off. */

/* Major Hook Ids */
#define CLLOCKD_HKID	0x91200000 /* Cluster Lock Manager */

/* Minor Hook Ids for cllockd */
#define T_STATS		0x0010
#define T_DEADLOCK	0x0020
#define T_CTI		0x0030
#define T_CLMLOCK	0x0040
#define T_UNIXLOCK	0x0050
#define T_LOCK		0x0060
#define T_RESOURCE	0x0070
#define T_BATCH		0x0080
#define T_AST		0x0090

/* Minor Hook Ids for all */
#define T_BASIC		0x0000	/* Basic Information */
#define T_ENTR_FUNC	0x0004	/* Enter a function */
#define T_EXIT_FUNC	0x0006	/* Exit a function */
#define T_INFO		0x0008 	/* General Info */

/* Cllockd Function */
#define T_CLLOCKD_CATCHER	0x00 /* cllockd catcher */
#define T_CLLOCKD_MAIN		0x01 /* cllockd main */
#define T_GROWLOCKTAB 		0x02 /* clm_alloc growlocktab */
#define T_PROCESS_OLD_REQUESTS	0x03 /* clm_batch process_old_requests */
#define T_PROCESS_QUEUE		0x04 /* clm_batch process_queue */
#define T_ADDCLIENT		0x05 /* clm_client addclient */
#define T_REMOVECLIENT		0x06 /* clm_client removeclient */
#define T_LOCAL_LOCK_CLM	0x07 /* clm_clmlock local_lock_clm */
#define T_CLM_LOCK		0x08 /* clm_clmlock clm_lock */
#define T_BEGIN_LOCK_CLM	0x09 /* clm_clmlock begin_lock_clm */
#define T_CONT_LOCK_CLM  	0x0A /* clm_clmlock cont_remote_lock_clm*/
#define T_CLM_CONVERT		0x0B /* clm_clmlock clm_convert */
#define T_CONVERT_LOCK_CLM	0x0C /* clm_clmlock convert_lock_clm */
#define T_CONVERT_LOCKS		0x0D /* clm_clmlock convert_locks */
#define T_BLOCKED_CONVERT	0x0E /* clm_clmlock blocked_convert */
#define T_BLOCKED_LOCK		0x0F /* clm_clmlock blocked_lock */
#define T_GRANT_LOCK		0x10 /* clm_clmlock grant_lock */
#define T_GRANT_CONVERT		0x11 /* clm_clmlock grant_convert */
#define T_SEND_BASTS		0x12 /* clm_clmlock send_basts */
#define T_SEND_BASTS_AGAIN	0x13 /* clm_clmlock send_basts_again */
#define T_SEND_BAST		0x14 /* clm_clmlock send_bast */
#define T_BEGIN_UNLOCK_CLM	0x15 /* clm_clmlock begin_unlock_clm */
#define T_CONT_UNLOCK_CLM 	0x16 /* cont_unlock_clm */
#define T_PRE_UNLOCK_CLM	0x17 /* clm_clmlock local_unlock_clm */
#define T_WAKE_ALL_LOCKS	0x18 /* clm_clmlock wake_all_locks */
#define T_WAKEUP_CLM		0x19 /* clm_clmlock wakeup_clm */
#define T_CLM_DUMP_RESOURCES	0x1A /* clm_clmlock clm_dump_resources */
#define T_CLM_RESET_VALUEBLOCKS	0x1B /* clm_reset_valueblocks */
#define T_PURGE_CLM		0x1C /* clm_clmlock purge_clm */
#define T_CLM_PROCESS		0x1D /* clm_cti clm_process */
#define T_CLM_SEND		0x1E /* clm_cti clm_send */
#define T_CLM_RECV		0x1F /* clm_cti clm_recv */
#define T_CLM_REQUEST		0x20 /* clm_cti clm_request */
#define T_CLM_REPLY		0x21 /* clm_cti clm_reply */
#define T_SENDUDP		0x22 /* clm_cti sendudp */
#define T_NOTIFY		0x23 /* clm_cti notify */
#define T_SENDAST		0x24 /* clm_cti sendast */
#define T_PROCESS_DEADLOCK_TIMEOUT_QUEUE 0x25 /* clm_deadlock ... */
#define T_PURGE_DEADLOCK	0x26 /* clm_deadlock purged_deadlock */
#define T_CLM_WRITE		0x27 /* clm_io clm_write */
#define T_CLM_READ		0x28 /* clm_io clm_read */
#define T_CLM_WRITEV		0x29 /* clm_io clm_writev */
#define T_PTI_MESSAGE		0x2A /* clm_libr pti_message */
#define T_PTI_CALL		0x2B /* clm_libr pti_call */
#define T_LOCAL_PURGE		0x2C /* clm_lock.c local_purge */
#define T_PURGELOCKS		0x2D /* clm_lock.c purgelocks */
#define T_REMOTE_PURGE		0x2E /* clm_lock.c remote_purge */
#define T_CONT_REMOTE_PURGE	0x2F /* clm_lock.c cont_remote_purge */
#define T_REMOTE_DEF_STALL	0x30 /* clm_lock.c remote_def_stall */
#define T_CONT_REMOTE_DEF_STALL	0x31 /* cont_remote_def_stall */
#define T_LOCAL_DEF_STALL	0x32 /* clm_lock.c local_def_stall */
#define T_REMOTE_STALL		0x33 /* clm_lock.c remote_stall */
#define T_CONT_REMOTE_STALL	0x34 /* clm_lock.c cont_remote_stall */
#define T_LOCAL_STALL		0x35 /* clm_lock.c local_stall */
#define T_REMOTE_UNSTALL	0x36 /* clm_lock.c remote_unstall */
#define T_CONT_REMOTE_UNSTALL	0x37 /* cont_remote_unstall */
#define T_LOCAL_UNSTALL		0x38 /* clm_lock.c local_unstall */
#define T_UNSTALL		0x39 /* clm_lock.c unstall */
#define T_CHECK_DEF_STALL	0x3A /* clm_lock.c check_def_stall */
#define T_PTI_PROG		0x3B /* clm_main.c pti_prog */
#define T_PURGE_LIST		0x3C /* clm_main.c purge_list */
#define T_KERN_MAIN		0x3D /* clm_main.c kern_main */
#define T_PTI_GET_REQUESTS	0x3E /* clm_main.c pti_get_requests */
#define T_PTI_GET_RESPONSES	0x3F /* clm_main.c pti_get_responses */
#define T_CLM_GETMSG		0x40 /* clm_main.c clm_getmsg */
#define T_FLUSH_ASYNC		0x41 /* clm_main.c flush_async */
#define T_SEND_ASYNC		0x42 /* clm_main.c send_async */
#define T_CLOSE_PRIMARY_SOCKET	0x43 /* clm_main.c close_primary_socket*/
#define T_ENTER_GRACE_PERIOD	0x44 /* clm_main.c enter_grace_period */
#define T_START_LOCKD		0x45 /* clm_main.c start_lockd */
#define T_UPDATE_NEW_PRIMARY	0x46 /* clm_main.c update_new_primary */
#define T_PROC_CLM_DUMPCOMPLETE 0x47 /* proc_clm_dumpcomplete */
#define T_STAT_CHANGE		0x48 /* clm_main.c stat_change */
#define T_STAT_CAST		0x49 /* clm_main.c stat_cast */
#define T_PROC_CLM_TOPCHG_NEW   0x4A /* clm_prot.c proc_clm_topchg_new*/
#define T_PROC_CLM_TOPCHG_FELLOFF 0x4B /* proc_clm_topchg_felloff */
#define T_PROC_CLM_CONFIG	0x4C /* clm_prot.c proc_clm_config */
#define T_GRACE_END		0x4D /* clm_prot.c grace_end */
#define T_CLM_STATS_T		0x4E /* clm_resource.c clm_stats_t */
#define T_REMOTE_REGISTER	0x4F /* clm_resource.c remote_register*/
#define T_CONT_REMOTE_REGISTER	0x50 /* cont_remote_register */
#define T_SEND_RESOURCES	0x51 /* clm_resource.c send_resources */
#define T_SEND_RESREFS		0x52 /* clm_resource.c send_resrefs */
#define T_CLM_PROG		0x53 /* clm_rpc.c clm_prog */
#define T_RPC_RESPOND		0x54 /* clm_rpc.c rpc_respond */
#define T_LKM_EXIT		0x55 /* clm_rpc.c lkm_exit */
#define T_LOCAL_LOCK_UNIX	0x56 /* clm_unixlock.c local_lock_unix*/
#define T_REMOTE_LOCK_UNIX	0x57 /* clm_unixlock.c remote_lock_unix*/
#define T_CONT_REMOTE_LOCK_UNIX	0x58 /* cont_remote_lock_unix */
#define T_LOCAL_UNLOCK_UNIX	0x59 /* local_unlock_unix */
#define T_REMOTE_UNLOCK_UNIX	0x5A /* remote_unlock_unix */
#define T_CONT_REMOTE_UNLOCK_UNIX 0x5B /* cont_remote_unlock_unix */
#define T_LOCAL_CANCEL_UNIX	0x5C /* local_cancel_unix */
#define T_REMOTE_CANCEL_UNIX    0x5D /* remote_cancel_unix */
#define T_CONT_REMOTE_CANCEL_UNIX 0x5E /* cont_remote_cancel_unix */
#define T_BLOCKED		0x5F /* clm_unixlock.c blocked */
#define T_CANCEL		0x60 /* clm_unixlock.c cancel */
#define T_WAKEUP_UNIX		0x61 /* clm_unixlock.c wakeup_unix */
#define T_PURGE_UNIX		0x62 /* clm_unixlock.c unix_purge */
#define T_EXIT			0x63 /* kernel.c exit */
#define T_ABORT			0x64 /* kernel.c abort */
#define T_CLLOCKD_FREE_LE	0x65 /* free_le */
#define T_CLLOCKD_GET_LE	0x66 /* get_le */
#define T_CLLOCKD_GROWLOCKTAB	0x67 /* growlocktab */
#define T_FIND_RECLOCK		0x68 /* find_reclock */
#define T_RLDB_EXPAND		0x69 /* rl_expand_freelist */
#define T_RLDB_RECLAIM		0x70 /* rl_reclaim_entry */
#define T_DIR_READ		0x71 /* process directory request */
#define T_DIRUPDT_READ		0x72 /* process directory update request */
#define T_MASTER_UNLOCK_CLM	0x80 /* master unlock routine */
#define T_LOCAL_UNLOCK_CLM	0x81 /* local unlock routine */
#define T_PROC_UPDATE		0x82 /* rldb update procedure */
#define T_UPDATE_DIR		0x83 /* directory update client procedure */

#define T_CCCP_OURID_BFR_LOOP	0x100 /* Check ourID */
#define T_CCCP_OURID_AFT_LOOP	0x101 /* Check ourID */
#define T_CCCP_OURID_ENT_LOOP	0x102 /* Check ourID */
#define T_CCCP_OURID_AFT_TOPO	0x103 /* Check ourID */
#define T_CCCP_OURID_GET_LOCAL	0x104 /* Check ourID */

#define T_RLDB_MALLOC		0x200 /* malloc error in rl_expand_freelist */
#define T_RLDB_ALLOC		0x201 /* freelist return from rl_alloc */
#define T_RLDB_FREE		0x202 /*  call to rl_free */
#define T_RLDB_INIT		0x203 /* rl_init called */
#define T_RLDB_CREATE		0x204 /* rl_create result */

#define T_MATCH_RESP		0x300 /* match response from master */
#define T_MISMATCH_RESP		0x301 /* didn't match response from master */

#define T_SENDAST_SUCC		0x400 /* AST successfully sent */
#define T_SENDBAST_SUCC		0x401 /* BAST successfully sent */

#endif /* !CLTRC_H */
