/*
 * Core modules for the IBM OSS Distributed Lock Manager
 *
 * Copyright (C) 2001, International Business Machines
 *
 * Authors: Peter R. Badovinatz, Ian D. Romanick
 * Contacts: wombat@us.ibm.com, idr@us.ibm.com
 * http://oss.software.ibm.com/developerworks/opensource/linux/projects/dlm
 *
 * This program is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY;  without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 * the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program;  if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
#ident "$Id: dlm_kernel.h,v 1.17 2001/11/07 23:57:09 idr Exp $"

/*
 *   COMPONENT_NAME: CLLOCKD
 *
 *   FUNCTIONS: FREE
 *		MALLOC
 *		REALLOC
 */

#ifndef _DLM_KERNEL_H
#define _DLM_KERNEL_H

#ifndef lint

#endif

/* ============================================================ */

#ifdef __KERNEL__

# include <linux/string.h>
# include <linux/types.h>
# include <linux/time.h>
# include <asm/semaphore.h>

# ifndef DECLARE_MUTEX_LOCKED
#  define DECLARE_MUTEX_LOCKED(n) struct semaphore n = MUTEX_LOCKED
# endif /* DECLARE_MUTEX_LOCKED */

#else

# include <stdlib.h>

#endif /*  __KERNEL__ */

#if !defined(ERROR)
#define ERROR (-1)
#endif

#ifdef __linux__

typedef unsigned char uchar;

#ifdef __KERNEL__
#define double error
#define float error
#endif /* __KERNEL__ */

#define FALSE 0
#define TRUE 1

#endif /* __linux__ */

typedef struct arg_struct {
  int debug;		/* debug mask */
  int local_debug;	/* local debug mask */
  int batch_sleep;	/* batch_sleep -not used- */
  int locksegsize;	/* lock table segment size */
  int ressegsize;	/* size of resource segments */
  int resprealloc;	/* number of res segs to preallocate */
  int hashsize;	/* number of record in the resource hash table */
  int	msg_key;	/* message queue key */
  int	kmem_cutoff;	/* percentage of remaining kmem to use */
  unsigned int recalc_time;	/* recalc time for migration */
  uint64_t decay_rate;      /* decay rate for migration */
  int  dlmdevicemajor;      /* Major number for DLM device */
  int  dlmadminminor;       /* Minor number for DLM "admin" port */
  int  dlmlocksminor;       /* Minor number for DLM "locks" port */
  char dlmconfigfile[128];  /* Absolute name of configuration file */
  char dumpfilename[128];	/* Absolute name of diagnostic dump file */
  char dlmdevicebase[64];   /* Base name of the DLM device */
  char dlmdevicename[128];  /* Absolute name of the DLM device */
  char dlmprocname[128];    /* DLM base dirname in /proc */
  char dlmclustermgr[128];  /* Cluster manager load module */
  char dlmadminbase[32];    /* DLM "admin" base name */
  char dlmadminname[128];   /* DLM "admin" absolute name */
  char dlmlocksbase[32];    /* DLM "locks" base name */
  char dlmlocksname[128];   /* DLM "locks" absolute name */
} arg_struct_t;

#ifdef __KERNEL__
extern arg_struct_t *args;
#else
extern arg_struct_t args;
#endif /* __KERNEL__ */

#ifdef __linux__
#include <stdarg.h>

/*
 * Provide a "bsdlog" that substitutes printk calls.  These
 * defines match the definitins (e.g., KERN_EMERG) for the
 * Linux printk defines.
 */

extern int bsdlog( int level, const char * fmt, ... )
    __attribute__ ((format (printf, 2, 3)));

extern int vbsdlog( int level, const char * fmt, va_list arglist );

#include <sys/syslog.h>

#endif /* __linux__ */

#ifdef __KERNEL__

#include <linux/slab.h>
#include <linux/vmalloc.h>

#define LOCK_FREE 0

#define DECLARE_DLM_SLEEPLOCK(n)         DECLARE_MUTEX(n)
#define DECLARE_DLM_SLEEPLOCK_LOCKED(n)  DECLARE_MUTEX_LOCKED(n)
#define DLM_SLEEPLOCK_INIT(l)     sema_init(l,1)
#define DLM_SLEEPLOCK_TRYLOCK(l)  (down_trylock(l) == 0)
#define DLM_SLEEPLOCK_LOCK(l)     down(l)
#define DLM_SLEEPLOCK_UNLOCK(l)   up(l)

#ifndef DLM_ALL_LOCKS_SLEEP
# define DECLARE_DLM_SPINLOCK(n)  spinlock_t n = SPIN_LOCK_UNLOCKED
# define DLM_SPINLOCK_INIT(l)     spin_lock_init(l)
# define DLM_SPINLOCK_TRYLOCK(l)  spin_trylock(l)
# define DLM_SPINLOCK_LOCK(l)     spin_lock(l)
# define DLM_SPINLOCK_UNLOCK(l)   spin_unlock(l)
#else
# define DECLARE_DLM_SPINLOCK(n)  DECLARE_DLM_SLEEPLOCK(n)
# define DLM_SPINLOCK_INIT(l)     DLM_SLEEPLOCK_INIT(l)
# define DLM_SPINLOCK_TRYLOCK(l)  DLM_SLEEPLOCK_TRYLOCK(l)
# define DLM_SPINLOCK_LOCK(l)     DLM_SLEEPLOCK_LOCK(l)
# define DLM_SPINLOCK_UNLOCK(l)   DLM_SLEEPLOCK_UNLOCK(l)
#endif


#ifdef USE_SIMPLE_LOCKS
typedef spinlock_t dlm_mutex_t;
# define DLM_MUTEX_INIT(l)     spin_lock_init(l)
# define DLM_MUTEX_TRYLOCK(l)  spin_trylock(l)
# define DLM_MUTEX_LOCK(l)     spin_lock(l)
# define DLM_MUTEX_UNLOCK(l)   spin_unlock(l)
#else
typedef struct semaphore dlm_mutex_t;
# define DLM_MUTEX_INIT(l)     sema_init(l,1)
# define DLM_MUTEX_TRYLOCK(l)  (down_trylock(l) == 0)
# define DLM_MUTEX_LOCK(l)     down(l)
# define DLM_MUTEX_UNLOCK(l)   up(l)
#endif /* USE_SIMPLE_LOCKS */

#ifdef __linux__
typedef dlm_mutex_t jlock_t;
#else
typedef struct jlock {
	dlm_mutex_t mutex ;
	pid_t	owner;
	int	level;
} jlock_t;
#endif /* __linux__ */
#endif /* __KERNEL__ */


#ifdef __KERNEL__

#define MALLOC(size)		my_malloc(size)
#define REALLOC(ptr,old,new)	my_realloc((ptr),(old),(new))
#define FREE(ptr)		my_free(ptr)

extern void * my_malloc(int);
extern unsigned int my_free( void * );

#else

#define MALLOC(size)		malloc(size)
#define REALLOC(ptr,old,new)	realloc((ptr),(old),(new))
#define FREE(ptr)		free(ptr)

#endif /* __KERNEL__ */

/* memory allocation macros used for memory checking */
#if defined(MEMCHK)
#define CLM_MALLOC(size)        bmalloc((size), __FILE__, __LINE__)
#define CLM_FREE(ptr)           bfree((ptr), __FILE__, __LINE__)
#else
#define CLM_MALLOC(size)        MALLOC((size))
#define CLM_FREE(ptr)           FREE((ptr))
#endif


#ifdef __KERNEL__
#ifdef __linux__

/*
 * Fake these out as well.
 */
int setuerror(int error);
int getuerror(void);
void *bsearch(const void *key,
               const void *base,
               size_t nmemb,
               size_t size,
               int(*compar)(const void *, const void *));
void *bsearchp(const void *key,
               const void *base,
               size_t nmemb,
               size_t size,
               int(*compar)(const void *, const void *));
char *printbin( const unsigned char * data, unsigned len );
int getpid(void);
int timeval_sub(struct timeval *t0,
                struct timeval *t1,
                struct timeval *t2);

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,10)
# define snprintf(buf,count,fmt, args...) \
	sprintf( buf, fmt, ## args )
# define vsnprintf(buf,count,fmt,args) \
	vsprintf( buf, fmt, args )
#endif

/*
 * Fake float stuff.
 */
int kfl_ftofi(/*uint64_t*/ unsigned long long int f) ;
char *kfl_ftos(/*uint64_t*/ unsigned long long int f) ;
/*uint64_t*/
unsigned long long int kfl_fitof(int i) ;
/*uint64_t*/
unsigned long long int kfl_mul(/*uint64_t*/ unsigned long long int a,
                               /*uint64_t*/ unsigned long long int b) ;
/*uint64_t*/
unsigned long long int kfl_add(/*uint64_t*/ unsigned long long int a,
                               /*uint64_t*/ unsigned long long int b) ;
/*uint64_t*/
unsigned long long int kfl_sub(/*uint64_t*/ unsigned long long int a,
                               /*uint64_t*/ unsigned long long int b) ;
/*uint64_t*/
unsigned long long int kfl_idivi(int i, int j) ;

/* 
 * an entry in the hash table
 */
struct hashent {
	struct hashent *next;
	int remoteid;
	int localid;
};

/*
 * The AIX kernel allows the dynamic creation of heaps, to
 * allow use of virtual memory within the kernel "outside"
 * of the kernel's memory range.  These heaps can be
 * paged out, quite a useful feature.
 *
 * But not one we can currently take advantage of.  However,
 * we still need to take some care about memory usage, and
 * track it, so that if we get out of control and threaten
 * the system, we can quit allocating locks.  So to most of
 * the DLM core we maintain the heap fiction.
 */

#include <asm/page.h>

struct heap {
  size_t amount;                        /* number of bytes used */
  size_t numpages;                      /* number of pages in heap */
  void  *start_addr;                    /* start of the "heap" */
  struct heap *next_heap;               /* next heap (if any) */
};

#ifndef PGSHIFT
#define PGSHIFT PAGE_SHIFT
#endif

#include <asm/unistd.h>
static __inline__ void _exit( int n )
{
    long  __res;


    __asm__ volatile( "int $0x80"
		      : "=a" (__res)
		      : "0" (__NR_exit), "b" ((long) n) );
}

#endif /* __linux__ */

#if defined(KERNEL_STATS)
enum {
	STAT_START, STAT_DISPATCH, STAT_SEND, STAT_RECEIVE, STAT_FINISH,
	NUMSTATS
};

struct sigma {
    int	seconds;
    int	microns;
    int	samples;
};
#endif

#ifndef BOOL_T_DEFINED
#define BOOL_T_DEFINED
typedef int bool_t;
#endif /* BOOL_T_DEFINED */

/* This appeared in linux/kernel.h sometime between 2.4.5 and 2.4.13.  This
 * implementation is taken exactly from linux/kernel.h in 2.4.13.
 */

#ifndef min

#define min(x,y) ({ \
        const typeof(x) _x = (x);       \
        const typeof(y) _y = (y);       \
        (void) (&_x == &_y);            \
        _x < _y ? _x : _y; })

#endif

/* Returns TRUE iff the specified node ID is part of the cluster.
 */

bool_t clm_node_reachable( int nodeid );

/* Returns the IP address to use to communicate with the specified node ID.
 */

ulong clm_id_to_addr( int nodeid );

/* Sets the name for the current kernel process.
 */

void dlm_set_thread_name( struct task_struct * task, const char * name );

/* Sends a signal to a user-mode process.
 */

int dlm_kill_user_proc( pid_t user_pid, int sig );

/* Returns a pointer to the name of the transaction type (see enum dlm_ops in
 * clmstructs.h).
 */

const char * transaction_type_string( int type );

/* Returns a pointer to the name of the lockn type (see enum dlm_restype in
 * dlm.h).
 */

const char * dlm_get_lock_type_string( unsigned type );

/*
 * Node ID <--> index conversion macros.  These are handy because as far as
 * the user is concerned, cluster nodes are numbered starting from 1.  The
 * cluster manager, on the other hand, indexes them starting from 0.
 *
 */

extern int index_to_nodeid(int idx);
extern int nodeid_to_index(int id);

#define INDEX_TO_NODEID(index)\
                       (index_to_nodeid(index))

#define NODEID_TO_INDEX(nodeid)\
                       (nodeid_to_index(nodeid))

/* Gets the local node's ID.
 */

int clm_getlocalid( int * id );

/* Return the number of configured nodes in the cluster.
 */

unsigned clm_node_count( void );


/* Returns TRUE if cluster configuration map is initialized.
 */

static __inline__ bool_t clm_config_inited( void )
{
    extern int OurSiteID;

    return( OurSiteID != -1 );
}



/* Creates and initializes a new dlm_nodeid_iterator object.  This type of
 * iterator will iterate over the node IDs that are currently part of the
 * cluster.  That is, each time iterator_next is called, the data pointer
 * will be set to the next node ID in the cluster.
 */

struct iterator * dlm_nodeid_iterator_create( void );

#endif /*  __KERNEL__ */

#endif /* _DLM_KERNEL_H */
