/*
 * Core modules for the IBM OSS Distributed Lock Manager
 *
 * Copyright (C) 2001, International Business Machines
 *
 * Authors: Peter R. Badovinatz, Ian D. Romanick
 * Contacts: wombat@us.ibm.com, idr@us.ibm.com
 * http://oss.software.ibm.com/developerworks/opensource/linux/projects/dlm
 *
 * This program is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY;  without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 * the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program;  if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * http://www.gnu.org/copyleft/lesser.html
 */

#ifndef ITERATOR_H
#define ITERATOR_H

#ident "$Id: iterator.h,v 1.2 2001/09/20 23:09:30 idr Exp $"

#ifndef BOOL_T_DEFINED
#define BOOL_T_DEFINED
typedef int bool_t;
#endif /* BOOL_T_DEFINED */

struct iterator;
typedef struct iterator iterator_t;


/*
 */

typedef bool_t (iterator_next_func)( iterator_t *, void ** );
typedef void (iterator_destroy_func)( iterator_t * );

struct iterator_methods
{
    iterator_destroy_func * destroy;
    iterator_next_func * next;
};


/* Abstract iterator type.  This is all that clients of an iterator need to
 * see, but concrete implementations will have more.
 */

struct iterator
{
    const struct iterator_methods * methods;
};




__inline__ static bool_t iterator_next( iterator_t * i, void ** data )
{
    return( i->methods->next( i, data ) );
}

__inline__ static void iterator_destroy( iterator_t * i )
{
    i->methods->destroy( i );
}

#endif /* ITERATOR_H */
