/*
 * Interface library for the IBM OSS Distributed Lock Manager
 *
 * Copyright (C) 2001, International Business Machines
 *
 * Authors: Peter R. Badovinatz, Ian D. Romanick
 * Contacts: wombat@us.ibm.com, idr@us.ibm.com
 * http://oss.software.ibm.com/developerworks/opensource/linux/projects/dlm
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * http://www.gnu.org/copyleft/lesser.html 
 */
#ident "$Id: api_data.c,v 1.5 2001/11/06 13:32:46 bkon Exp $"

/*
 *   COMPONENT_NAME: DLM LIB
 *
 *   FUNCTIONS: defined
 */
#ifndef API_REAL_DATA_H
#define API_REAL_DATA_H

/* ============================================================ */

/*
 * OurPid and ClientInit work together, in that the former is
 * used as the client's canonical id to identify it to the DLM,
 * and ClientInit specifies whether or not the client is
 * initialized.  We need two variables since we cannot otherwise
 * identify among the multiple threads in a multi-thread client.
 * See ClmInit() in api_init.c for more info.
 * FOR KERNEL API: We use 0x7F7F7F7F (outside the range of valid pids, 
 * but positive) as OurPid because we 
 * don't have the thread vs. process problems, don't send signals in the    
 * kernel and want to allow multiple kernel threads to work as a group.
 */

#ifdef __KERNEL__
#include "clmstructs.h"
int OurPid = DLM_KERNEL_PID;		/* dummy process id of kernel process */
#else
int OurPid = 0;		/* process id of current process */
int ClientInit = 0; /* has this client initialized with DLM? */
#endif /*__KERNEL__*/

int debug = 0;		/* api debug flag */
int transdebug = 0;	/* api transaction debug flag */
unsigned int lvb_old_behave;	/* flag to control LVB behavior - CR960021 */
#ifndef _THREAD_SAFE 
int dlm_errno;	/* global status variable */
#endif
int local_debug; 	/* may be assigned by application source code */
int notify_signo = 0;	/* signal sent to client for AST notification */

#endif /* API_REAL_DATA_H */
