/*
 * Interface library for the IBM OSS Distributed Lock Manager
 *
 * Copyright (C) 2001, International Business Machines
 *
 * Authors: Peter R. Badovinatz, Ian D. Romanick
 * Contacts: wombat@us.ibm.com, idr@us.ibm.com
 * http://oss.software.ibm.com/developerworks/opensource/linux/projects/dlm
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * http://www.gnu.org/copyleft/lesser.html 
 */
#ident "$Id: api_purge.c,v 1.4 2001/11/06 13:32:47 bkon Exp $"

/*
 *   COMPONENT_NAME: DLM LIB
 *
 *   FUNCTIONS: MSGSTR
 *		clm_purge
 */

#ifdef __KERNEL__

# ifndef MODULE
#  define MODULE
# endif
 
# include <linux/kernel.h>
# define __NO_VERSION__
# include <linux/module.h>
# ifdef USING_FULL_VERSIONING
#  if CONFIG_MODVERSIONS==1
#   define MODVERSIONS
#   include <linux/modversions.h>
#  endif
# endif /* USING_FULL_VERSIONING */
 
#else


#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/file.h>
#include <sys/stat.h>

#endif /*__KERNEL__*/

#include "dlm.h"
#include "ast.h"
#include "clmstructs.h"
#include "pti.h"

#ifdef HAVING_NLS_ON_LINUX
/* for NLS */
#include "general_msg.h"
#define MSGSTR(Num, Str) catgets(get_catd("general.cat"), GENERAL, Num, Str)
#else   /* HAVING_NLS_ON_LINUX */
#define MSGSTR(Num, Str) Str
#endif  /* HAVING_NLS_ON_LINUX */
/* end of NLS */

#include "api_data.h"
#include "api_proto.h"  /* Should be LAST included file. */

/* ============================================================ */
/*
 * Name
 *	dlm_purge
 * Description
 *	purge all locks associated with site:pid
 * Arguments
 *	site - cluster site id of process to purge
 *	pid - process id of process to purge - 0 matches any pid
 *	flags - purge flags as defined in "purged.h"
 * Returns
 *	DLM_BADARGS for illegal site id
 *	dlm_errno as set by Dlm_Init, api_reqresp, api_response
 *	else status as returned from lockmanager
 * Global Data Affected
 *	dlm_errno
 */
dlm_stats_t
dlm_purge(int site, int pid, int flags __attribute__ ((__unused__)) )
{
	struct transaction request;
	struct transaction response;
	int ret_val;		/* value to be returned by function */
	
	/* dlm_errno set by Dlm_Init */
	if ((ret_val = Dlm_Init()) != DLM_NORMAL) {
		goto done;
	}

	request.clm_purgelocks.purgesite = site;
	request.clm_purgelocks.purgepid = pid;
	request.clm_purgelocks.pid = OurPid;
	request.clm_purgelocks.flags = 0;
	request.clm_respflags = RESP_UDP;

retry:
	
	if (api_reqresp(DLM_PURGELOCKS,&request,&response) != 0) {
		ret_val = dlm_errno;
		goto done;
	}

	if (dlm_errno == DLM_NORMAL && 
		response.clm_status == DLM_DENIED_GRACE_PERIOD) {
#if defined(DEBUG)
if (debug) {
	fprintf(stderr, "dlm_purge: grace sleep(%d)\n", GRACE_SLEEP);
}
#endif
		api_sleep(GRACE_SLEEP);
		goto retry;
	}

	ret_val = response.clm_status;

done:

	return ret_val;
}
	
