/*
 * Interface library for the IBM OSS Distributed Lock Manager
 *
 * Copyright (C) 2001, International Business Machines
 *
 * Authors: Peter R. Badovinatz, Ian D. Romanick
 * Contacts: wombat@us.ibm.com, idr@us.ibm.com
 * http://oss.software.ibm.com/developerworks/opensource/linux/projects/dlm
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * http://www.gnu.org/copyleft/lesser.html 
 */
#ident "$Id: api_resource.c,v 1.3 2001/11/29 13:20:22 bkon Exp $"

/*
 *   COMPONENT_NAME: DLM LIB
 *
 *   FUNCTIONS: register_resource
 */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/file.h>
#include <sys/stat.h>

#include "dlm.h"
#include "ast.h"
#include "clmstructs.h"
#include "pti.h"

#include "api_data.h"
#include "api_proto.h"  /* Should be LAST included file. */

/* ============================================================ */
/*
 * Name
 *	register_resource
 * Description
 *	Send a resource registration request to the lock manager
 *	and wait for a response.
 * Arguments 
 *	name - NULL terminated resource name
 * 	type - resource type
 * Returns
 *	a resource handle for success
 *	NULL for failure - with clm_errno set
 */
union dlm_rh
register_resource(char *name,
                  dlm_restype_t type)
{
	struct transaction request;
	struct transaction response;
	short namelen;

	request.clm_respflags = RESP_UDP;
	response.clm_handle.rh = 0;
#if defined(DEBUG)
if (debug) {
	fprintf(stderr, "dlmregister(%s, %d)\n", name, type);
}
#endif
	/* clm_errno set by Clm_Init */
	if (Dlm_Init() != DLM_NORMAL)
		goto done;

	if (type >= DLM_RES_NTYPES) {
		set_dlm_errno(DLM_BADTYPE);
		goto done;
	}

	namelen = strlen(name);
	if (namelen < 1 || namelen > MAXRESOURCELEN) {
		set_dlm_errno(DLM_IVBUFLEN);
		goto done;
	}

	request.clm_register.name =
			request.clm_register.unix_name;
        memcpy(request.clm_register.name, name, namelen);
	request.clm_register.namelen = namelen;
	request.clm_register.type = type;
	request.clm_register.pid = OurPid;
	request.clm_register.count = 1;
	request.clm_register.rh.rh = 0;
	if (!api_reqresp(DLM_REGISTER, &request,&response)) {
		/* SaveHandle removed */
		if (dlm_errno == DLM_NORMAL && response.clm_handle.rh == 0)
			set_dlm_errno(DLM_MAXHANDLES);
	} else
		response.clm_handle.rh = 0;

done:
#if defined(DEBUG)
if (debug) {
	fprintf(stderr, "dlmregister() returning 0x%X\n", 
		response.clm_handle.rh);
}
#endif
	return response.clm_handle;
}
