/*
 * Interface library for the IBM OSS Distributed Lock Manager
 *
 * Copyright (C) 2001, International Business Machines
 *
 * Authors: Peter R. Badovinatz, Ian D. Romanick
 * Contacts: wombat@us.ibm.com, idr@us.ibm.com
 * http://oss.software.ibm.com/developerworks/opensource/linux/projects/dlm
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * http://www.gnu.org/copyleft/lesser.html 
 */
#ident "$Id: api_scn.c,v 1.4 2001/11/29 13:20:22 bkon Exp $"

/*
 *   COMPONENT_NAME: DLM LIB
 *
 *   FUNCTIONS: MSGSTR
 *		clm_scnop
 */

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/file.h>
#include <sys/stat.h>

#include "dlm.h"
#include "ast.h"
#include "clmstructs.h"
#include "pti.h"

#ifdef HAVING_NLS_ON_LINUX
/* for NLS */
#include "general_msg.h"
#define MSGSTR(Num, Str) catgets((struct _catalog_descriptor*)get_catd("general.cat"), GENERAL, Num, Str)
#else   /* HAVING_NLS_ON_LINUX */
#define MSGSTR(Num, Str) Str
#endif  /* HAVING_NLS_ON_LINUX */
/* end of NLS */

#include "api_data.h"
#include "api_proto.h"  /* Should be LAST included file. */

/* ============================================================ */
dlm_stats_t
dlm_scnop(	int		lockid,		/* lockid to use */
		scn_op_t	op_type,	/* scn operation type */
		short		bit_len,	/* bit length of SCN */
		char		*in_lvb,	/* input LVB */
		char		*out_lvb)	/* output LVB */
{
	struct transaction request;
	struct scninfo *vi = &request.clm_scninfo;
	struct transaction response;
	int ret_val;	/* value to be returned from function */
	int reconfig = 0;


	/* check clinfo initialization */
	/* dlm_errno set by Dlm_Init */
	if ((ret_val = Dlm_Init()) != DLM_NORMAL) {
		goto done;
	}

        memset(&request, 0, sizeof(request));

	/* do we need an input value? */
	if (op_type >= SCN_ADD && op_type <= SCN_SET) {

		/* did they provide one? */
		if (NULL == in_lvb) {
			return DLM_BADARGS;
		} else {
                       memcpy(vi->lkvalue, in_lvb, sizeof(vi->lkvalue));
		}
	} else if (op_type < SCN_CUR || op_type > SCN_SET) {
		ret_val = set_dlm_errno(DLM_BADARGS);
		goto done;
	}

	/* check for valid bit length and output pointer */
	if (bit_len < 1 || bit_len > MAX_SCN_SIZE || (!out_lvb)) {
		ret_val = set_dlm_errno(DLM_BADARGS);
		goto done;
	}

	/* set up request for lock manager */
	request.clm_locktype = DLM_RES_VMS;
	request.clm_respflags = RESP_UDP;
	request.clm_pid = OurPid;
	vi->pid = OurPid;
	vi->bit_len = bit_len;
	vi->lockid = lockid;
	vi->scn_op = op_type;
retry:
	if (api_reqresp(DLM_SCN_OP, &request, &response)) {
		ret_val = set_dlm_errno(DLM_NOLOCKMGR);
		goto done;
	}

	if (response.clm_status == DLM_WORKING) {
		printf(MSGSTR(DLM_WORKING,
                      "DLM_WORKING_TER returned from API call!!\n"));
		if (api_response(&response) != 0) {
			ret_val = set_dlm_errno(DLM_NOLOCKMGR);
			goto done;
		}
	}

	if (dlm_errno == DLM_NORMAL && 
		response.clm_status == DLM_DENIED_GRACE_PERIOD) {
#if defined(DEBUG)
if (debug) {
	fprintf(stderr, "clmunlock: grace sleep(%d)\n", GRACE_SLEEP);
}
#endif
		reconfig = 1;
		api_sleep(GRACE_SLEEP);
		goto retry;
	}

	ret_val = set_dlm_errno(response.clm_status);
	if (ret_val == DLM_NORMAL)
		memcpy(out_lvb, response.clm_scninfo.lkvalue,
				sizeof(response.clm_scninfo.lkvalue)); 

done:

	return ret_val;
}
