/*
 * Interface library for the IBM OSS Distributed Lock Manager
 *
 * Copyright (C) 2001, International Business Machines
 *
 * Authors: Peter R. Badovinatz, Ian D. Romanick
 * Contacts: wombat@us.ibm.com, idr@us.ibm.com
 * http://oss.software.ibm.com/developerworks/opensource/linux/projects/dlm
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * http://www.gnu.org/copyleft/lesser.html 
 */
#ident "$Id: api_setnotify.c,v 1.2 2001/03/12 19:55:44 wombat Exp $"

/*
 *   COMPONENT_NAME: DLM LIB
 *
 *   FUNCTIONS: dlm_setnotify
 */

#include <stdio.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/file.h>
#include <sys/stat.h>

#include "dlm.h"
#include "ast.h"
#include "clmstructs.h"
#include "pti.h"
#include "clmutex_r.h"

#include "api_data.h"

#ifdef  _THREAD_SAFE
static pthread_mutex_t notifysigno_mutex = PTHREAD_MUTEX_INITIALIZER;
#endif

/* ============================================================ */
/*
 * Name
 *	dlm_setnotify
 * Description
 *	This routine specifies a signal to be delivered when an
 *	AST has been delivered. Upon delivery of all subsequent
 *	ASTs, the lock manager will send the specified signal to 
 *	lock client. If this routine is never called, the lock
 *	manager will not deliver the signal. If a signal number
 *	of 0 is specified, signal notification of AST delivery is 
 *	disabled.
 * Arguments
 *	signo - the signal to be delivered or 0 to disable
 *		(see <sys/signal.h>)
 *	oldsigp - if non-null, the existing signal number is returned
 *		into the specified location.
 * Returns
 *	DLM_NORMAL for success
 *	DLM_BADARGS for negative or out-of-range signal number
 */
dlm_stats_t 
dlm_setnotify(int sig, int *oldsigp)
{
	if (sig < 1 || sig > NSIG)
		return DLM_BADARGS;

	TS_LOCK(&notifysigno_mutex);
	if (oldsigp != NULL)
		*oldsigp = notify_signo;
	notify_signo = sig;
	TS_UNLOCK(&notifysigno_mutex);

	return DLM_NORMAL;
}
