/*
 * Interface library for the IBM OSS Distributed Lock Manager
 *
 * Copyright (C) 2001, International Business Machines
 *
 * Authors: Peter R. Badovinatz, Ian D. Romanick
 * Contacts: wombat@us.ibm.com, idr@us.ibm.com
 * http://oss.software.ibm.com/developerworks/opensource/linux/projects/dlm
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * http://www.gnu.org/copyleft/lesser.html 
 */
#ident "$Id: api_syscall.c,v 1.4 2001/06/04 18:11:38 idr Exp $"

/*
 *   COMPONENT_NAME: DLM LIB
 *
 *   FUNCTIONS: api_recvpkt
 *		api_sendpkt
 */

#include <sys/types.h>
#include <errno.h>

#include "dlm.h"
#include "ast.h"
#include "clmstructs.h"
#include "pti.h"

#include "api_proto.h"  /* Should be LAST included file. */

/* ============================================================ */
/*
 * Function:
 *	api_sendpkt
 *
 * Description:
 *	"Send" a transaction to the kernel for processing.  A loop is used so
 * that we can retry after being interrupted within the kernel by a signal.
 *
 * Arguments:
 *	char *p;		-- pointer to transaction
 *	int len;		-- length of transaction
 *
 * Return value:
 *	 0 on sucess
 *	<0 on error
 *
 * Side effects:
 *	NONE
 */

int
api_sendpkt (void *p, unsigned int len)
{
    int status;

    do
    {
	status = clm_send(p,len);
    } while ( status > 0 );

    return( status );
}

/*
 * Function:
 *	api_recvpkt
 *
 * Description:
 *	"Receive" a transaction response from the kernel.  A loop is used so
 * that we can retry after being interrupted within the kernel by a signal.
 *
 * Arguments:
 *	char *p;		-- pointer to transaction
 *	int len;		-- length of transaction
 *	int seq;		-- sequence # for which we expect a response
 *
 * Return value:
 *	 0 on sucess
 *	<0 on error
 *
 * Side effects:
 *	NONE
 */

int
api_recvpkt (void *p, unsigned int len, int seq)
{
    int status;

    do
    {
	status = clm_recv(p,len,seq);
    } while ( status > 0 );

    return( status );
}
