/*
 * Interface library for the IBM OSS Distributed Lock Manager
 *
 * Copyright (C) 2001, International Business Machines
 *
 * Authors: Peter R. Badovinatz, Ian D. Romanick
 * Contacts: wombat@us.ibm.com, idr@us.ibm.com
 * http://oss.software.ibm.com/developerworks/opensource/linux/projects/dlm
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * http://www.gnu.org/copyleft/lesser.html 
 */
#ident "$Id: ast_stub.c,v 1.6 2001/09/05 18:27:54 idr Exp $"

/*
 *   COMPONENT_NAME: DLM LIB
 *
 *   FUNCTIONS: ASTpoll
 */

#include "dlm.h"
#include "ast.h"
#include "clmstructs.h"
#include "pti.h"
#include "clmutex_r.h"
#include "ast_stub.h"


/*
 * Name
 *	ASTpoll
 * Description
 * 	Invoke any pending ASTs from the client side
 * Arguments
 *	pid - process ID of client
 *	tid - thread ID of client
 * Returns
 * 	Number of ASTs serviced
 * Global Data Affected
 *	???	
 */

int
ASTpoll( int pid __attribute__ ((__unused__)), 
	 int tid __attribute__ ((__unused__)) ) 
{
	int nast;
	struct poll_info pi;

	/* Obviously, the kernel does 99.44% of the work here */
	for (nast = 0; ASTget(&pi) > 0; ++nast) {
		if (pi.pi_type == AST_LOCK) {
			(*pi.pi_func)( pi.pi_args, pi.pi_mode, NULL );
		} else {
			(*pi.pi_func)(pi.pi_args, pi.pi_status, pi.pi_extrap);
		}
	}

	return(nast);
}
