/*
 * Interface library for the IBM OSS Distributed Lock Manager
 *
 * Copyright (C) 2001, International Business Machines
 *
 * Authors: Peter R. Badovinatz, Ian D. Romanick
 * Contacts: wombat@us.ibm.com, idr@us.ibm.com
 * http://oss.software.ibm.com/developerworks/opensource/linux/projects/dlm
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * http://www.gnu.org/copyleft/lesser.html 
 */
#ident "$Id: clmutex_r.h,v 1.2 2001/11/06 13:32:47 bkon Exp $"

/* define thread_safe macro or kernel spinlock if kernel api    */
#ifdef __KERNEL__
    #define TS_LOCK(lock)     spin_lock(lock)
    #define TS_UNLOCK(lock)   spin_unlock(lock)
#else /* __KERNEL__ */
    #ifdef  _THREAD_SAFE
        #include <pthread.h>

        #ifdef  DEBUG
            #include <assert.h>
            #define TS_LOCK(lock)   (assert(pthread_mutex_lock(lock) == 0))
            #define TS_TRYLOCK(lock) (!(pthread_mutex_trylock(lock) == EBUSY))
            #define TS_UNLOCK(lock) (assert(pthread_mutex_unlock(lock) == 0))
       #else  /* DEBUG */
            #define TS_LOCK(lock)   (pthread_mutex_lock(lock))
            #define TS_TRYLOCK(lock) (!(pthread_mutex_trylock(lock) == EBUSY))
            #define TS_UNLOCK(lock) (pthread_mutex_unlock(lock))
       #endif  /* DEBUG */
    #else  /* _THREAD_SAFE */
        #define TS_LOCK(lock)
        #define TS_TRYLOCK(lock)    1
        #define TS_UNLOCK(lock)
    #endif   /*_THREAD_SAFE */
#endif /*__KERNEL__*/
