/*
 * Interface library for the IBM OSS Distributed Lock Manager
 *
 * Copyright (C) 2001, International Business Machines
 *
 * Authors: Peter R. Badovinatz, Ian D. Romanick
 * Contacts: wombat@us.ibm.com, idr@us.ibm.com
 * http://oss.software.ibm.com/developerworks/opensource/linux/projects/dlm
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * http://www.gnu.org/copyleft/lesser.html 
 */
#ident "$Id: print.c,v 1.3 2001/07/31 23:54:34 idr Exp $"

/*
 *   COMPONENT_NAME: DLM LIB
 *
 *   FUNCTIONS: dlm_errmsg
 *		dlm_perror
 */

#include <stdio.h>

#include "dlm.h"

#ifdef HAVING_NLS_ON_LINUX
/* for NLS */
#include "general_msg.h"
#define MSGSTR(Num, Str) catgets((struct _catalog_descriptor*)get_catd("general.cat"), GENERAL, Num, Str)
#else   /* HAVING_NLS_ON_LINUX */
#define MSGSTR(Num, Str) Str
#endif  /* HAVING_NLS_ON_LINUX */
/* end of NLS */

/* ============================================================ */

extern char *dlm_msgtab[];
extern int dlm_nstats;

/*
 * return a string representation of an error code
 */
char *
dlm_errmsg( dlm_stats_t err )
{
	static char buffer[50];
	if (err < 0 || err  >= dlm_nstats) {
		sprintf(buffer,
                MSGSTR(DLM_ERROR,"DLM Error %d\n"),
                err);
		return buffer;
	}
	return dlm_msgtab[err];
}

/*
 * equivalent of UNIX perror for lock manager dlm_errno
 */
void
dlm_perror(const char *s)
{
	fprintf(stderr,
            MSGSTR(ERROR_MSG,"%1$s: %2$s\n"),
            s,
            dlm_errmsg(dlm_errno));
}
