/*
 * Sample locking client (using cancel)
 *
 * Copyright (C) 2001, International Business Machines
 *
 * Authors: Peter R. Badovinatz, Ian D. Romanick
 * Contacts: wombat@us.ibm.com, romanick@us.ibm.com
 * http://oss.software.ibm.com/developerworks/projects/dlm
 *
 * This program is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY;  without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 * the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program;  if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

#ident "$Id: cancel1.c,v 1.2 2001/08/31 22:54:15 wombat Exp $"

/* ============================================================ */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>

#include <sys/types.h>
#include <sys/wait.h>

#include <dlm.h>

/* ============================================================ */

struct lockstatus lksb[12];        /* array of lock status blocks */
static int ASTexecuted = 0;
static const char * progname = "";

static pid_t child_pid = 0;
static void child( void );
/*static void ast_func( void * astarg );*/
static dlm_astlockfunc_t ast_func;

#define LOCK_NAME "TSTLCK"

/* ============================================================ */

void
error_exit( const char * msg, int status )
{
    fprintf( stderr, "%s: %s\n", dlm_errmsg( status ), msg );
    
    if ( child_pid != 0 )
    {
	waitpid( child_pid, NULL, 0 );
    }

    exit( 1 );
}

int
main( int argc, char ** argv )
{
    dlm_stats_t   status;
    int astarg = 1;

    progname = argv[0];

    child_pid = fork();
    if ( child_pid == 0 )
    {
	child();
    }

    
    /* Wait for a few seconds so that the child has time to get the lock.
     * I wish there was a cleaner way to do this.
     */

    printf( "parent pre-waiting\n" );
    sleep( 10 );

    status = dlmlock( LKM_PWMODE, & lksb[astarg], LKM_VALBLK, LOCK_NAME,
		      strlen( LOCK_NAME ), ast_func, &astarg, NULL );
    if ( status != DLM_NORMAL )
    {
	error_exit( "second lock", status );
    }

    
    /* Wait for a few seconds and make sure that we don't aquire the lock.
     */

    printf( "parent waiting\n" );
    sleep( 5 );

    
    /* Check the status of the second lock.  If the status is NOT blocked,
     * then we have a problem.
     */

    printf( "status is %s, and the AST routine was %scalled\n",
	    dlm_errmsg( lksb[astarg].status ),
	    (ASTexecuted == 0) ? "not " : "" );
    if ( (lksb[astarg].status != DLM_BLOCKED) && (ASTexecuted != 0) )
    {
	error_exit( "should be blocked", lksb[1].status );
    }


    /* Since the lock is blocked, cancel the request.
     */

    printf( "parent canceling lock\n" );
    status = dlmunlock_sync( lksb[astarg].lockid,
                             NULL,
                             LKM_INVVALBLK | LKM_CANCEL );
    if ( status != DLM_NORMAL )
    {
	error_exit( "first unlock", status );
    }

    waitpid( child_pid, NULL, 0 );
    return( 0 );
}




void child( void )
{
    dlm_stats_t   status;


    /* Get the lock and sit on it for a bit.
     */

    status = dlmlock_sync( LKM_PWMODE, & lksb[0], LKM_VALBLK, LOCK_NAME,
		      strlen( LOCK_NAME ), NULL, NULL );
    if ( status != DLM_NORMAL )
    {
	error_exit( "first lock", status );
    }
    
    printf( "child has lock\n" );
    sleep( 60 );


    printf( "child releasing lock\n" );
    status = dlmunlock_sync( lksb[0].lockid,
                             NULL,
                             LKM_INVVALBLK );
    if ( status != DLM_NORMAL )
    {
	error_exit( "second unlock", status );
    }

    exit( 0 );
}

/* ------------------------------------------------------------ */
/*
 * Normal AST routine that is triggered by ASTpoll.
 */
void ast_func(void *astarg)
{
    char * msg;
    int * astptr = (int *)astarg;
    int   astval = *astptr;

    msg = dlm_errmsg( lksb[astval].status );
    printf( "[%s:%s] status/astarg/id[%s/%d(%x)]\n",
            progname, __FUNCTION__, msg,
            astval, lksb[astval].lockid );
    ASTexecuted = 1;
}
