/*
 * Sample locking client
 *
 * Copyright (C) 2001, International Business Machines
 *
 * Authors: Peter R. Badovinatz, Ian D. Romanick
 * Contacts: wombat@us.ibm.com, idr@us.ibm.com
 * http://oss.software.ibm.com/developerworks/projects/dlm
 *
 * This program is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY;  without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 * the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program;  if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
#ident "$Id: client1.c,v 1.1 2001/03/12 23:11:21 wombat Exp $"

/* ============================================================ */
/*
 * This program is a very simple locking client.  Idea is to
 * basically just make a pass through a lock request and
 * handle an AST return.
 *
 * Code for now is essentially just a cut-and-paste from the
 * client programming manual.
 */
/* ============================================================ */

#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>

#include "../source/include/dlm.h"

struct lockstatus lksb[12];   /* array of lock status blocks */
int which_lock;               /* index into array of lock status blocks */
void ast_func();              /* AST routine */
void sig_func();              /* signal handling routine */

dlm_stats_t status;
void * stat;
char *msg; /* for printable status code */
extern int transdebug;

int
main(int argc, char *argv[])
{
  int astarg = 0;             /* astarg parameter */

  transdebug = 1;

  status = dlm_setnotify( SIGUSR1, NULL );
  if( status != DLM_NORMAL ) {
    dlm_perror("dlm_setnotify");
  }
  stat = signal( SIGUSR1, sig_func );
  if( SIG_ERR == stat ) {
    perror("signal");
  }
  which_lock = 0;
  astarg = which_lock;
  printf("Calling dlmlock!\n");
  status = dlmlock( LKM_CRMODE,         /* mode */
                    &lksb[which_lock],  /* lock status block */
                    LKM_VALBLK,
                    "RES-A",            /* name */
                    5,                  /* namelen */
                    ast_func,           /* ast routine */
                    &astarg,            /* astargs */
                    0);                 /* bast */
  if ( status != DLM_NORMAL ) {
    dlm_perror( "clmlock");
  } else {
    printf("Return DLM_NORMAL from clmlock!\n");
  }

  exit(status);
}

/* Signal handling routine; calls ASTpoll to trigger AST routine */
void sig_func()
{
  ASTpoll( getpid(), 0 );
}

/* Routine that is triggered by ASTpoll. */
void ast_func(int *astarg)
{
  msg = dlm_errmsg( lksb[*astarg].status );
  printf("status= %s; astarg passed = %d",msg,*astarg);
}
