/*
 * Sample locking client
 *
 * Copyright (C) 2001, International Business Machines
 *
 * Authors: Peter R. Badovinatz, Ian D. Romanick, Beth Kon
 * Contacts: wombat@us.ibm.com, idr@us.ibm.com, bkon@us.ibm.com
 * http://oss.software.ibm.com/developerworks/projects/dlm
 *
 * This program is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY;  without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 * the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program;  if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
#ident "$Id: kclient2.c,v 1.1 2001/11/08 14:17:06 bkon Exp $"

/* ============================================================ */
/*
 * This program is a very simple locking client for the kernel API.  Idea is to
 * basically just make a pass through a lock request and
 * handle an AST return.
 *
 * Non-interactive client module:
 *  - Parameters to specify at load time:
 *    - requested_lock_mode (one of "NL", "CR", "CW", "PR", "PW", "EX","ALL".
 *      Mode of the lock. "ALL" means run the test repeatedly for each
 *      of the 6 modes.
 *    - requested_wait_time (default 10 secs): after lock is granted, 
 *        how long to wait before unlocking it.
 *    - display_quiet - if set to one, suppresses some printks.
 */
/* ============================================================ */
#ifndef __KERNEL__
#define __KERNEL__
#endif
#ifndef MODULE
#define MODULE
#endif

#include <linux/kernel.h>
#include <asm/errno.h>
#include <asm/unistd.h>
#include <linux/string.h>
#include <linux/wait.h>
#include <linux/sched.h>
#include <linux/smp_lock.h>
#define __NO_VERSION__                  /* Only dlm_kerndd.c sets version. */
#include <linux/module.h>
#include <linux/version.h>              /* Due to __NO_VERSION__ */
#ifdef USING_FULL_VERSIONING
#if CONFIG_MODVERSIONS==1
#define MODVERSIONS
#include <linux/modversions.h>
#endif
#endif /* USING_FULL_VERSIONING */


#include <dlm.h>

/* ============================================================ */

char *requested_lock_mode=NULL;
int requested_wait_time=10;
int display_quiet=0;
MODULE_PARM(requested_lock_mode, "s");
MODULE_PARM(requested_wait_time, "i");
MODULE_PARM(display_quiet, "i");

struct lockstatus lksb[12];        /* array of lock status blocks */
int which_lock;                    /* index into array of lock status blocks */
dlm_astlockfunc_t ast_func;                   /* AST routine */
dlm_astunlockfunc_t ast_unlock_func;                   /* AST routine */
dlm_bastlockfunc_t bast_func;                  /* AST routine */

dlm_stats_t status;
void * stat;
char * msg;                        /* for printable status code */
int    ASTexecuted;
int    UNLOCKASTexecuted;


/* ------------------------------------------------------------ */


static char * lock_mode_tab[] =
{
  "NL", "CR", "CW", "PR", "PW", "EX", "ALL"
} ;

uint desired_wait_time;
int desired_lock_mode;
char desired_lock_name[MAXDLMNAMELEN] = "RES-A";

/* ============================================================ */
/*
 * Print usage message.
 */
static void
print_usage(void)
{
  printk("****************************************************************\n");
  printk("DLM kernel API test program usage:\n");
  printk("modprobe convert.o [requested_lock_mode=lock_mode] [requested_wait_time=wait_seconds] [display_quiet=[1|0]]\n");
  printk(" lock_mode: one of \"NL\", \"CR\", \"CW\", \"PR\", \"PW\", \"EX\", \"ALL\".\n");
  printk("            default value - \"CR\"\n");
  printk("   ***********************************\n");
  printk(" wait_seconds: maximum time to wait for AST to be returned,\n");
  printk("               default value - 10,\n");
  printk("               0 (zero) means wait forever if necessary!\n");
  printk("   ***********************************\n");
  printk(" display_quiet: if set to one, suppresses informational printk\n");
  printk("****************************************************************\n");

  return;
}

/*
 * Function:
 *      clm_sleep
 *
 * Description:
 *      Sleep for the specified number of seconds, but check the
 *      nsleep return for signal interruptions and ignore all but KILL
 *
 * Arguments:
 *      number of seconds to sleep
 *
 * Return value:
 *      Not currently used.
 *
 * Side effects:
 *      NONE
 *
 */
void
clm_sleep(int   secs)
{
    long   wakeup;
    DECLARE_WAIT_QUEUE_HEAD( wait );


    /* The sleep call returns the number of jiffies left to sleep from
     * the original timeout.
     */
    wakeup = secs * HZ;

    init_waitqueue_head( & wait );
    do
    {
      /* BUGBUG We should just hit the signal handler if any hit, unless
       * BUGBUG they're blocked...  If they are blocked, do we even care
       * BUGBUG to get them?
       */
#if 0
      if (signal_pending()) {
        sig_handle(current->sigpending);
      }
#endif
        wakeup = interruptible_sleep_on_timeout( & wait, wakeup );
    } while( wakeup > 0 );
}
/*================================================================*/
int
kclient2_main()
{
  int astarg = 0;             /* astarg parameter */
  int waiting;
  int unlock_flag;
  int lock_mode_start;
  int lock_mode_finish;
  int i;

  ASTexecuted = 0;
  UNLOCKASTexecuted = 0;
 

  sprintf(current->comm,"kclient2_main"); /* comm is 16 bytes */
  lock_kernel();   /* This seems to be required for exit_mm */
  exit_mm(current);
  unlock_kernel();  
                   /* close open files too (stdin/out/err are open) */
/*

* can't get this to work!!!!!can't find proper include file so I can't load.
  for (i=255; i>=0; i--)
       if (current->files->fd[i])
             close(i);
*/


  which_lock = 0;
  astarg = which_lock;
  if (LKM_MAXMODE + 1 != desired_lock_mode) {
          lock_mode_start = desired_lock_mode;
          lock_mode_finish = desired_lock_mode +1;
  } else {
          lock_mode_start = 0;
          lock_mode_finish = LKM_MAXMODE + 1;
  }
  for (i = lock_mode_start; i < lock_mode_finish; i++) {
          printk(KERN_INFO __FUNCTION__ "Call dlmlock mode/name/astarg[%s/%s/%d]!\n",
                 desired_lock_name,
                 lock_mode_tab[i],
                 astarg);
          status = dlmlock( i,  /* mode */
                            &lksb[which_lock],  /* lock status block */
                            LKM_VALBLK,
                            desired_lock_name,  /* name */
                            (strlen(desired_lock_name)), /* namelen */
                            ast_func,           /* ast routine */
                            &astarg,            /* astargs */
                            bast_func);         /* bast */
          if ( status != DLM_NORMAL ) {
            printk(KERN_INFO __FUNCTION__ "dlmlock call with lock mode %s returned status %d\n",
                    lock_mode_tab[i], status);
            break;
          } else {
            printk(KERN_INFO __FUNCTION__ "Return DLM_NORMAL from dlmlock, lksb.status = %d!\n",
                   lksb[which_lock].status);
          }

          waiting = desired_wait_time;
          if ( ! display_quiet )
          {
                 printk( KERN_INFO __FUNCTION__ "Sleeping %d secs:", waiting );
          }

            clm_sleep(waiting);

         if ( ! display_quiet )
         {
            printk("\n");
         }

          if (ASTexecuted) {
            printk(KERN_INFO __FUNCTION__ "AST block executed!\n");
            ASTexecuted = 0;
          } else {
            printk(KERN_INFO __FUNCTION__ "AST block not executed after %d second%s\n",
                   desired_wait_time,
                   ((1 == desired_wait_time) ? "!" : "s!"));
          }
        
          if (DLM_NORMAL == lksb[which_lock].status) {
            unlock_flag = LKM_FORCE|LKM_INVVALBLK;
          } else {
            unlock_flag = LKM_CANCEL|LKM_INVVALBLK;
          }
          printk(KERN_INFO __FUNCTION__ "Call dlmunlock id/name/flag[%d(%x)/%s/%x]!\n",
                 lksb[which_lock].lockid,
                 lksb[which_lock].lockid,
                 desired_lock_name,
                 unlock_flag);
          status = dlmunlock( lksb[which_lock].lockid,  /* id of lock */
                              NULL,
                              unlock_flag, /* release the lock */
                              ast_unlock_func,           /* ast routine */
                              &astarg,            /* astargs */
                              &lksb[which_lock]);    
          if ( status != DLM_NORMAL ) {
            printk("KERN_INFO __FUNCTION__ dlmunlock call with lock mode %s returned status %d\n",
                    lock_mode_tab[i], status);
            break;
          } else {
            printk(KERN_INFO __FUNCTION__ "Return DLM_NORMAL from dlmunlock!\n");
          }

          waiting = desired_wait_time;
          if ( ! display_quiet )
          {
                 printk(KERN_INFO __FUNCTION__ "Sleeping %d secs:", waiting );
          }

            clm_sleep(waiting);

         if ( ! display_quiet )
         {
            printk("\n");
         }

          if (UNLOCKASTexecuted) {
            printk(KERN_INFO __FUNCTION__ "UNLOCK AST block executed!\n");
            printk(KERN_INFO __FUNCTION__ "TEST FINISHED SUCCESSFULLY\n");
            UNLOCKASTexecuted = 0;
          } else {
            printk(KERN_INFO __FUNCTION__ "UNLOCK AST block not executed after %d second%s\n",
                   desired_wait_time,
                   ((1 == desired_wait_time) ? "!" : "s!"));
          }


     }
    return(0);
}


/* ------------------------------------------------------------ */

/* ------------------------------------------------------------ */
/*
 * Parse command line and set the locking flags to be used.
 *
 * Return
 *  0 - no errors, global values saved as desired.
 *  !0 - errors, printed on stderr
 */
int
init_module(void)
{
  int ret = 0;
  int i, pid,good=0;
  /*EXPORT_NO_SYMBOLS;

  clm_sleep(10);*/

  desired_wait_time = (uint)requested_wait_time;
  if ( NULL == requested_lock_mode) {
      desired_lock_mode=LKM_CRMODE;
  } else {
      for (i = 0, good = 0; i <= LKM_MAXMODE + 1; i++) {
            if (0 == strcmp(requested_lock_mode, lock_mode_tab[i])) {
              desired_lock_mode = i;
              good = 1;
              break;
            }
          }
          if (!good) {
            printk(KERN_INFO __FUNCTION__ " Invalid lock mode given [%s]!\n",
	           	  requested_lock_mode );
            print_usage();
            ret = -EINVAL;
          }
  }

  if ( (0 == ret) && ! display_quiet ) {
    printk(KERN_INFO __FUNCTION__ "Given options:\n");
    printk(KERN_INFO __FUNCTION__ "\tMode[%s] Name[%s] Waittime[%d].\n",
           lock_mode_tab[desired_lock_mode],
           desired_lock_name,
           desired_wait_time);
  }
pid = kernel_thread(kclient2_main, NULL, 0);
    if (-1 == pid) {
        printk(KERN_INFO __FUNCTION__ "unable to spawn kernel thread\n");
    }

  return(ret);
}

/* ============================================================ */

/*
 * Normal AST routine that is triggered by ASTpoll.
 */
void ast_func(void *astarg)
{
 int *astval;
 astval = (int *)astarg;
 

/* ORIGINAL STUFF. dlm_errmsg not yet functional for kernel 
* msg = dlm_errmsg( lksb[*astval].status );
*/
 printk(KERN_INFO __FUNCTION__ "[%s] status/astarg/id[%s/%d/%d(%x)]\n",
         __FUNCTION__,
         "dlm_errmsg not yet supported",   /* this line used to say msg*/
         *astval,
         lksb[*astval].lockid,
         lksb[*astval].lockid);

  ASTexecuted = 1;
}

/* ------------------------------------------------------------ */
/*
 * Normal AST routine that is triggered by ASTpoll.
 */
void ast_unlock_func(void *astarg, int lstat, void * extrap)
{
 int *astval;
 astval = (int *)astarg;
 

/* ORIGINAL STUFF. dlm_errmsg not yet functional for kernel 
* msg = dlm_errmsg( lksb[*astval].status );
*/
 printk("KERN_INFO __FUNCTION__ [%s] status/astarg/id[%s/%d/%d(%x)]\n",
         __FUNCTION__,
         "dlm_errmsg not yet supported",   /* this line used to say msg*/
         *astval,
         lksb[*astval].lockid,
         lksb[*astval].lockid);
  UNLOCKASTexecuted = 1;
}
/* ------------------------------------------------------------ */
/*
 * Blocking AST routine that is triggered by ASTpoll.
 */
void bast_func(void *astarg, int mode)
{
  int *astval;
  astval = (int *)astarg;

/* ORIGINAL STUFF. dlm_errmsg not yet functional for kernel 
* msg = dlm_errmsg( lksb[*astval].status );
*/
  printk(KERN_INFO __FUNCTION__ "In bast_func. [%s] status= %s; astval passed = %d; mode = %s\n",
         __FUNCTION__,
         msg,
         *astval,
         lock_mode_tab[mode]);
  ASTexecuted = 1;
}
